
file_in = '/Users/songweizhi/Desktop/to_be_p/tmp/GCA_013203245.1_data_report.tsv'
file_out = '/Users/songweizhi/Desktop/to_be_p/tmp/GCA_013203245.1_data_report_fmt.tsv'


def get_nonredundant_table(file_in, file_out, ignore_na):

    col_dict = dict()
    col_list = []
    col_index_dict = dict()
    line_num_index = 0
    for each_line in open(file_in):
        line_num_index += 1
        line_split = each_line.strip().split('\t')
        if line_num_index == 1:
            col_list = line_split
            col_index_dict = {key: i for i, key in enumerate(line_split)}
        else:
            for col in col_list:
                if col not in col_dict:
                    col_dict[col] = [line_split[col_index_dict[col]]]
                else:
                    col_dict[col].append(line_split[col_index_dict[col]])

    # write out
    file_out_handle = open(file_out, 'w')
    for col in col_list:

        value_list_uniq = list(set(col_dict[col]))
        if value_list_uniq == ['']:
            value_list_uniq = ['na']

        if col not in ['Assembly BioSample Attribute Name', 'Assembly BioSample Attribute Value']:
            if ignore_na is True:
                if value_list_uniq != ['na']:
                    file_out_handle.write('%s:\t%s\n' % (col, ','.join(value_list_uniq)))
            else:
                file_out_handle.write('%s:\t%s\n' % (col, ','.join(value_list_uniq)))
        else:
            biosample_attribute_name_list  = col_dict.get('Assembly BioSample Attribute Name', [])
            biosample_attribute_value_list = col_dict.get('Assembly BioSample Attribute Value', [])
            for (name, value) in zip(biosample_attribute_name_list, biosample_attribute_value_list):
                file_out_handle.write('Assembly BioSample Attribute %s:\t%s\n' % (name, value))
    file_out_handle.close()


get_nonredundant_table(file_in, file_out, ignore_na=True)




#!/usr/bin/env python3
import os
import glob
import shutil
import argparse


CheckM_usage = '''
====================================== CheckM example commands ======================================

# only parse quality file
BioSAK CheckM -i checkm_op.txt -o MAG_qualities.txt
BioSAK CheckM -i checkm_op.txt -o MAG_qualities.txt -r

# get the quality of qualified bins 
BioSAK CheckM -i checkm_op.txt -cpl 99 -o MAG_qualities_cpl99.txt
BioSAK CheckM -i checkm_op.txt -cpl 99 -ctm 5 -o MAG_qualities_cpl99_ctm5.txt

# get the quality of qualified MAG and copy them into a separate folder
BioSAK CheckM -i checkm_op.txt -g MAG_folder -x fa -cpl 99 -o MAG_qualities_cpl99.txt
BioSAK CheckM -i checkm_op.txt -g MAG_folder -x fa -cpl 99 -ctm 5 -o MAG_qualities_cpl99_ctm5.txt

# Note
adjusted_contamination = contamination x (100 - heterogeneity) / 100

=====================================================================================================
'''


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


def CheckM(args):

    input_quality_file = args['i']
    output_file =        args['o']
    input_bin_folder =   args['g']
    bin_file_extension = args['x']
    cpl_cutoff =         args['cpl']
    ctm_cutoff =         args['ctm']
    recalculate_ctm =    args['r']

    qualitfied_bin_folder = None
    if input_bin_folder is not None:

        if input_bin_folder[-1] == '/':
            input_bin_folder = input_bin_folder[:-1]

        input_bin_folder_no_path = input_bin_folder
        if '/' in input_bin_folder:
            input_bin_folder_no_path = input_bin_folder.split('/')[-1]

        if (cpl_cutoff is not None) and (ctm_cutoff is None):
            qualitfied_bin_folder = '%s_cpl%s' % (input_bin_folder_no_path, cpl_cutoff)

        if (cpl_cutoff is None) and (ctm_cutoff is not None):
            qualitfied_bin_folder = '%s_ctm%s' % (input_bin_folder_no_path, ctm_cutoff)

        if (cpl_cutoff is not None) and (ctm_cutoff is not None):
            qualitfied_bin_folder = '%s_cpl%s_ctm%s' % (input_bin_folder_no_path, cpl_cutoff, ctm_cutoff)

    # create output folder
    if (input_bin_folder is not None) and ((cpl_cutoff is not None) or (ctm_cutoff is not None)):
        if os.path.isdir(qualitfied_bin_folder) is True:
            print('Output MAG folder detected, program exited!')
            print(qualitfied_bin_folder)
            exit()
        os.mkdir(qualitfied_bin_folder)

    qualified_bin_num = 0
    output_file_handle = open(output_file, 'w')
    if recalculate_ctm is False:
        output_file_handle.write('Genome\tCompleteness\tContamination\tHeterogeneity\n')
    else:
        output_file_handle.write('Genome\tCompleteness\tContamination\tHeterogeneity\tAdjusted_contamination\n')
    for each_line in open(input_quality_file):

        if (not each_line.startswith('-')) and (not each_line.startswith('  Bin')):
            quality_split = each_line.strip().split(' ')
            quality_split_new = []
            for each_line in quality_split:
                if each_line != '':
                    quality_split_new.append(each_line)

            genome_id     = quality_split_new[0]
            completeness  = float(quality_split_new[12])
            contamination = float(quality_split_new[13])
            heterogeneity = float(quality_split_new[14])

            if cpl_cutoff is None:
                cpl_cutoff = 0

            if ctm_cutoff is None:
                ctm_cutoff = 9999

            if recalculate_ctm is False:
                if (completeness >= cpl_cutoff) and (contamination <= ctm_cutoff):
                    output_file_handle.write('%s\t%s\t%s\t%s\n' % (genome_id, completeness, contamination, heterogeneity))
                    qualified_bin_num += 1
                    if input_bin_folder is not None:
                        os.system('cp %s/%s.%s %s/' % (input_bin_folder, genome_id, bin_file_extension, qualitfied_bin_folder))
            else:
                ctm_adjusted = float("{0:.2f}".format(contamination*(100-heterogeneity)/100))
                if (completeness >= cpl_cutoff) and (ctm_adjusted <= ctm_cutoff):
                    output_file_handle.write('%s\t%s\t%s\t%s\t%s\n' % (genome_id, completeness, contamination, heterogeneity, ctm_adjusted))
                    qualified_bin_num += 1
                    if input_bin_folder is not None:
                        os.system('cp %s/%s.%s %s/' % (input_bin_folder, genome_id, bin_file_extension, qualitfied_bin_folder))
    output_file_handle.close()

    # report
    if (cpl_cutoff is not 0) or (ctm_cutoff is not 999):
        print('The number of qualified genomes: %s' % qualified_bin_num)


if __name__ == "__main__":

    CheckM_parser = argparse.ArgumentParser(usage=CheckM_usage)
    CheckM_parser.add_argument('-i',    required=True,                       help='CheckM produced quality file')
    CheckM_parser.add_argument('-o',    required=True,                       help='output quality file (reformatted)')
    CheckM_parser.add_argument('-g',    required=False,                      help='MAG folder')
    CheckM_parser.add_argument('-x',    required=False, default='fasta',     help='bin file extension')
    CheckM_parser.add_argument('-cpl',  required=False, type=float,          help='completeness cutoff (0-100)')
    CheckM_parser.add_argument('-ctm',  required=False, type=float,          help='contamination cutoff (0-100)')
    CheckM_parser.add_argument('-r',    required=False, action="store_true", help='recalculate contamination')
    args = vars(CheckM_parser.parse_args())
    CheckM(args)



import argparse
from Bio import SeqIO


def get_single_file_length(sequence_file):

    total_length_bp = 0
    for each_seq in SeqIO.parse(sequence_file, 'fasta'):
        total_length_bp += len(each_seq.seq)

    total_length_Mbp = float("{0:.2f}".format(total_length_bp / (1024 * 1024)))

    return total_length_Mbp


def get_multi_file_length():
    pass





parser = argparse.ArgumentParser()

parser.add_argument('-seq', required=True, help='sequence file')

args = vars(parser.parse_args())
sequence_file = args['seq']

total_length_bp = 0
for each_seq in SeqIO.parse(sequence_file, 'fasta'):
    total_length_bp += len(each_seq.seq)

total_length_Mbp = float("{0:.2f}".format(total_length_bp/(1024*1024)))

print('The total length of sequences in file %s is  %s in Mbp' % (sequence_file, total_length_Mbp) )





import os
import glob
import argparse
from ete3 import Tree
import multiprocessing as mp
from BioSAK.BioSAK_config import config_dict

compare_trees_usage = '''
================ compare_trees example command ================

module load R/3.5.3
BioSAK compare_trees -t1 tree_1.newick -t2 tree_2.newick
BioSAK compare_trees -t1 tree_dir -t2 tree_dir -tx newick -dm -t 12

===============================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def check_numeric(str_in):
    is_numeric = True
    try:
        x = float(str_in)
    except ValueError:
        is_numeric = False

    return is_numeric


def parse_mantel_stats(mantel_stats_txt):

    mantel_similarity = 'na'
    for each_line in open(mantel_stats_txt):
        if 'Mantel statistic r: ' in each_line:
            mantel_similarity = each_line.strip().split('Mantel statistic r: ')[1]
    return mantel_similarity


def get_matrix(query_tree_list, subject_tree_list, mantel_stats_dir, write_out_dm, output_matrix, output_matrix_distance):

    header_line_str = '\t' + '\t'.join(subject_tree_list) + '\n'

    output_matrix_handle = open(output_matrix, 'w')
    output_matrix_handle.write(header_line_str)
    distance_lol = []
    for each_qt in query_tree_list:

        current_qt_mantel_stats_value_list = [each_qt]
        for each_st in subject_tree_list:

            qt_vs_st_mantel_stats = '%s/%s_vs_%s_mantel_stats.txt' % (mantel_stats_dir, each_qt, each_st)
            st_vs_qt_mantel_stats = '%s/%s_vs_%s_mantel_stats.txt' % (mantel_stats_dir, each_st, each_qt)

            tree_similarity = 'na'
            if os.path.isfile(qt_vs_st_mantel_stats) is True:
                tree_similarity = parse_mantel_stats(qt_vs_st_mantel_stats)
            if os.path.isfile(st_vs_qt_mantel_stats) is True:
                tree_similarity = parse_mantel_stats(st_vs_qt_mantel_stats)

            current_qt_mantel_stats_value_list.append(tree_similarity)

        current_qt_mantel_stats_value_list_distance = [each_qt]
        for each_value in current_qt_mantel_stats_value_list[1:]:
            if check_numeric(each_value) is True:
                in_distance = 1 - float(each_value)
                in_distance = float("{0:.4f}".format(in_distance))
                if in_distance == 0:
                    in_distance = '0'
                current_qt_mantel_stats_value_list_distance.append(str(in_distance))
            else:
                current_qt_mantel_stats_value_list_distance.append('na')

        distance_lol.append(current_qt_mantel_stats_value_list_distance)
        current_qt_mantel_stats_value_str = '\t'.join(current_qt_mantel_stats_value_list)
        output_matrix_handle.write(current_qt_mantel_stats_value_str + '\n')
    output_matrix_handle.close()

    # write out distance matrix
    if write_out_dm is True:
        output_matrix_distance_handle = open(output_matrix_distance, 'w')
        output_matrix_distance_handle.write(header_line_str)
        for each_list in distance_lol:
            output_matrix_distance_handle.write('\t'.join(each_list) + '\n')
        output_matrix_distance_handle.close()


def compare_trees_worker(arg_list):

    compare_trees_R = arg_list[0]
    tree_file_1     = arg_list[1]
    tree_file_2     = arg_list[2]
    keep_tmp_file   = arg_list[3]

    tree1_path, tree1_basename, tree1_extension = sep_path_basename_ext(tree_file_1)
    tree2_path, tree2_basename, tree2_extension = sep_path_basename_ext(tree_file_2)

    op_stats = '%s_vs_%s_mantel_stats.txt' % (tree1_basename, tree2_basename)

    t1 = Tree(tree_file_1, format=1)
    t2 = Tree(tree_file_2, format=1)

    tree1_leaf_list = []
    for leaf1 in t1:
        tree1_leaf_list.append(leaf1.name)

    tree2_leaf_list = []
    for leaf2 in t2:
        tree2_leaf_list.append(leaf2.name)

    shared_leaves = set(tree1_leaf_list).intersection(tree2_leaf_list)
    if len(shared_leaves) == 0:
        print('No leaves shared between %s and %s, calculation skipped!' % (tree1_basename, tree2_basename))
        #exit()

    elif len(tree1_leaf_list) == len(tree2_leaf_list) == len(shared_leaves):
        compare_trees_cmd = 'Rscript %s -a %s -b %s > %s' % (compare_trees_R, tree_file_1, tree_file_2, op_stats)
        os.system(compare_trees_cmd)

    elif (len(shared_leaves) != len(tree1_leaf_list)) or (len(shared_leaves) != len(tree2_leaf_list)):
        print('Performing Mantel test based on %s leaves shared by %s (%s) and %s (%s)' % (len(shared_leaves), tree1_basename, len(tree1_leaf_list), tree2_basename, len(tree2_leaf_list)))

        # write out shared leaves
        shared_leaves_txt = '%s_vs_%s_shared_leaves.txt' % (tree1_basename, tree2_basename)
        shared_leaves_txt_handle = open(shared_leaves_txt, 'w')
        for each_shared_leaf in shared_leaves:
            shared_leaves_txt_handle.write(each_shared_leaf + '\n')
        shared_leaves_txt_handle.close()

        # subset_tree
        t1_subset     = '%s_vs_%s_%s_subset%s' % (tree1_basename, tree2_basename, tree1_basename, tree1_extension)
        t2_subset     = '%s_vs_%s_%s_subset%s' % (tree1_basename, tree2_basename, tree2_basename, tree2_extension)
        subset_cmd_t1 = 'BioSAK subset_tree -tree %s -taxon %s -out %s -q' % (tree_file_1, shared_leaves_txt, t1_subset)
        subset_cmd_t2 = 'BioSAK subset_tree -tree %s -taxon %s -out %s -q' % (tree_file_2, shared_leaves_txt, t2_subset)
        os.system(subset_cmd_t1)
        os.system(subset_cmd_t2)

        compare_trees_cmd = 'Rscript %s -a %s -b %s > %s' % (compare_trees_R, t1_subset, t2_subset, op_stats)
        os.system(compare_trees_cmd)

        if keep_tmp_file is False:
            os.system('rm %s' % shared_leaves_txt)
            os.system('rm %s' % t1_subset)
            os.system('rm %s' % t2_subset)


def compare_trees(args):

    compare_trees_R = config_dict['compare_trees_R']
    #output_prefix   = args['p']
    tree_file_1     = args['t1']
    tree_file_2     = args['t2']
    tree_file_ext   = args['tx']
    export_dm       = args['dm']
    num_threads     = args['t']
    keep_tmp        = args['tmp']

    query_tree_list = []
    if os.path.isfile(tree_file_1):
        query_tree_list = [tree_file_1]
    elif os.path.isdir(tree_file_1):
        query_tree_re = '%s/*.%s' % (tree_file_1, tree_file_ext)
        query_tree_list = glob.glob(query_tree_re)

    subject_tree_list = []
    if os.path.isfile(tree_file_2):
        subject_tree_list = [tree_file_2]
    elif os.path.isdir(tree_file_2):
        subject_tree_re = '%s/*.%s' % (tree_file_2, tree_file_ext)
        subject_tree_list = glob.glob(subject_tree_re)

    # prepare arg list for compare_trees_worker
    to_be_calculated_set = set()
    list_for_compare_trees_worker = []
    for each_query_tree in query_tree_list:
        for each_subject_tree in subject_tree_list:

            tree_1_vs_2 = '%s_vs_%s' % (each_query_tree, each_subject_tree)
            tree_2_vs_1 = '%s_vs_%s' % (each_subject_tree, each_query_tree)

            if tree_1_vs_2 not in to_be_calculated_set:
                list_for_compare_trees_worker.append([compare_trees_R, each_query_tree, each_subject_tree, keep_tmp])
                to_be_calculated_set.add(tree_1_vs_2)
                to_be_calculated_set.add(tree_2_vs_1)

    print('Total pairs of trees to compare: %s' % len(list_for_compare_trees_worker))

    # compare trees with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(compare_trees_worker, list_for_compare_trees_worker)
    pool.close()
    pool.join()

    # get matrix
    output_matrix_similarity    = 'Matrix_similarity.txt'
    output_matrix_distance      = 'Matrix_distance.txt'
    query_tree_list_basename = []
    for each_q_tree in query_tree_list:
        q_tree_path, q_tree_basename, q_tree_ext = sep_path_basename_ext(each_q_tree)
        query_tree_list_basename.append(q_tree_basename)

    subject_tree_list_basename = []
    for each_s_tree in subject_tree_list:
        s_tree_path, s_tree_basename, s_tree_ext = sep_path_basename_ext(each_s_tree)
        subject_tree_list_basename.append(s_tree_basename)

    get_matrix(sorted(query_tree_list_basename), sorted(subject_tree_list_basename), '.', export_dm, output_matrix_similarity, output_matrix_distance)

    # final report
    if export_dm is True:
        print('Data matrix exported to: %s and %s' % (output_matrix_similarity, output_matrix_distance))
    else:
        print('Data matrix exported to: %s' % output_matrix_similarity)

    print('Done!')


if __name__ == '__main__':

    compare_trees_parser = argparse.ArgumentParser(usage=compare_trees_usage)
    compare_trees_parser.add_argument('-t1',  required=True,                       help='tree (folder) 1')
    compare_trees_parser.add_argument('-t2',  required=True,                       help='tree (folder) 2')
    compare_trees_parser.add_argument('-tx',  required=False, default='newick',    help='extention of tree files, default: newick')
    compare_trees_parser.add_argument('-dm',  required=False, action="store_true", help='export distance-alike matrix, obtained by subtract the similarity value from 1')
    compare_trees_parser.add_argument('-t',   required=False, type=int, default=1, help='number of threads')
    compare_trees_parser.add_argument('-tmp', required=False, action="store_true", help='keep tmp files')
    args = vars(compare_trees_parser.parse_args())
    compare_trees(args)



import os
import glob
import argparse
import multiprocessing as mp


prokka_usage = '''
======================== prokka example commands ========================

BioSAK prokka -i gnm_folder -x fa -t 12 -d genome_domain.txt
BioSAK prokka -i mag_folder -x fa -t 12 -m genome_type.txt

# gnm_domain.txt (no file extension, tab separated)
# genomes not included in gnm_domain.txt will be treated as "Bacteria"
gnm_1   Archaea
gnm_2   Bacteria
   
# genome_type.txt (no file extension, tab separated)
# genomes not included in gnm_type.txt will be treated as "nonMAG"
gnm_1   MAG
gnm_2   nonMAG

=========================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(file_name)
    return f_path, f_base, f_ext


def prokka(args):

    gnm_dir         = args['i']
    gnm_ext         = args['x']
    gnm_domain_txt  = args['d']
    meta_mode_txt   = args['m']
    execute_cmd     = args['e']
    num_threads     = args['t']

    prokka_cmds_txt = '%s_prokka_cmds.txt' % gnm_dir

    # read in gnm domain info
    gnm_domain_dict = dict()
    if os.path.isfile(gnm_domain_txt) is True:
        for each_line in open(gnm_domain_txt):
            each_line_split = each_line.strip().split('\t')
            gnm_id = each_line_split[0]
            gnm_domain = each_line_split[1]
            if gnm_domain not in ['Archaea', 'Bacteria']:
                print('please specify either Archaea or Bacteria')
                exit()
            gnm_domain_dict[gnm_id] = gnm_domain

    # read in gnm type info (MAG or not)
    gnm_type_dict = dict()
    if os.path.isfile(meta_mode_txt) is True:
        for each_line in open(meta_mode_txt):
            each_line_split = each_line.strip().split('\t')
            gnm_id = each_line_split[0]
            gnm_type = each_line_split[1]
            if gnm_type not in ['MAG', 'nonMAG']:
                print('please specify either MAG or nonMAG')
                exit()
            gnm_type_dict[gnm_id] = gnm_type

    gnm_file_re   = '%s/*.%s' % (gnm_dir, gnm_ext)
    gnm_file_list = glob.glob(gnm_file_re)

    prokka_cmd_list = []
    for each_gnm in gnm_file_list:
        _, gnm_id, _ = sep_path_basename_ext(each_gnm)
        gnm_domain = gnm_domain_dict.get(gnm_id, 'Bacteria')
        gnm_type = gnm_type_dict.get(gnm_id, 'nonMAG')

        prokka_cmd = 'prokka --force --compliant --cpus 1 --kingdom %s --prefix %s --locustag %s --strain %s --outdir %s_prokka_wd %s' % (gnm_domain, gnm_id, gnm_id, gnm_id, gnm_id, each_gnm)
        if gnm_type == 'MAG':
            prokka_cmd = 'prokka --force --compliant --metagenome --cpus 1 --kingdom %s --prefix %s --locustag %s --strain %s --outdir %s_prokka_wd %s' % (gnm_domain, gnm_id, gnm_id, gnm_id, gnm_id, each_gnm)
        prokka_cmd_list.append(prokka_cmd)

    # write out command
    prokka_cmds_txt_handle = open(prokka_cmds_txt, 'w')
    for each_cmd in prokka_cmd_list:
        prokka_cmds_txt_handle.write(each_cmd + '\n')
    prokka_cmds_txt_handle.close()

    # run prokka with mp
    if execute_cmd is True:
        print('Annotating %s genomes with %s cores' % (len(prokka_cmd_list), num_threads))
        pool = mp.Pool(processes=num_threads)
        pool.map(os.system, prokka_cmd_list)
        pool.close()
        pool.join()
        print('Done!')


if __name__ == '__main__':

    parser = argparse.ArgumentParser(usage=prokka_usage)
    parser.add_argument('-i',          required=True,                          help='genome folder')
    parser.add_argument('-x',          required=True, default='fna',           help='file extension, deafult: fna')
    parser.add_argument('-d',          required=True,                          help='genome domain, Bacteria or Archaea')
    parser.add_argument('-m',          required=False, action="store_true",    help='annotate MAG')
    parser.add_argument('-e',          required=False, action="store_true",    help='execute commands')
    parser.add_argument('-t',          required=False, type=int, default=1,    help='number of threads')
    args = vars(parser.parse_args())
    prokka(args)



import os
import argparse
from Bio import AlignIO


SliceMSA_usage = '''
========================= SliceMSA example commands =========================

BioSAK SliceMSA -i 16S_aln.fasta -s 200-300 -o 16S_aln_200-300.fasta
BioSAK SliceMSA -i 16S_aln.phylip -fi phylip-relaxed -s sections.txt -o SliceMSA_op -fo phylip-relaxed

# example
200-300     select columns 200-300
-100        select columns 1-300
500-        select columns from 500 to the end

# Example of sections.txt (one section per line):
200-300
-100
500-

# Examples of alignment format (https://biopython.org/wiki/AlignIO): 
fasta, phylip, phylip-relaxed, phylip-sequential, clustal

=============================================================================
'''


def msa2fasta(msa_object, fasta_out):

    with open(fasta_out, 'w') as fasta_out_handle:
        for each_seq in msa_object:
            fasta_out_handle.write('>%s\n' % each_seq.id)
            fasta_out_handle.write('%s\n' % str(each_seq.seq))


def msa2phylip(msa_object, phylip_out):

    max_seq_id_len = 0
    for each_seq in msa_object:
        seq_id_len = len(each_seq.id)
        if seq_id_len > max_seq_id_len:
            max_seq_id_len = seq_id_len

    with open(phylip_out, 'w') as phylip_out_handle:
        phylip_out_handle.write('%s %s\n' % (len(msa_object), msa_object.get_alignment_length()))
        for each_seq in msa_object:
            seq_id = each_seq.id
            seq_id_with_space = '%s%s' % (seq_id, ' ' * (max_seq_id_len + 2 - len(seq_id)))
            phylip_out_handle.write('%s%s\n' % (seq_id_with_space, str(each_seq.seq)))


def SliceMSA(args):

    msa_in_file         = args['i']
    aln_in_format       = args['fi']
    col_to_select_txt   = args['s']
    op_dir              = args['o']
    aln_out_format      = args['fo']
    force_overwriting   = args['force']

    aln_out_ext = 'fasta'
    if aln_out_format == 'phylip-relaxed':
        aln_out_ext = 'phylip'

    if os.path.isfile(msa_in_file) is False:
        print('Input MSA not found, program exited!')
        exit()

    # read in msa
    msa_in = AlignIO.read(msa_in_file, aln_in_format)

    # parse provided sections
    section_to_select_list = []
    if os.path.isfile(col_to_select_txt) is False:
        col_to_select_txt_split = col_to_select_txt.strip().split('-')
        if col_to_select_txt == '-':
            section_to_select_list.append(['1', str(msa_in.get_alignment_length())])
        elif col_to_select_txt.startswith('-'):
            section_to_select_list.append(['1', col_to_select_txt_split[1]])
        elif col_to_select_txt.endswith('-'):
            section_to_select_list.append([col_to_select_txt_split[0], str(msa_in.get_alignment_length())])
        else:
            section_to_select_list.append(col_to_select_txt_split)
    else:
        for each_section in open(col_to_select_txt):
            each_section = each_section.strip()
            each_section_split = each_section.strip().split('-')
            if each_section == '-':
                section_to_select_list.append(['1', str(msa_in.get_alignment_length())])
            elif each_section.startswith('-'):
                section_to_select_list.append(['1', each_section_split[1]])
            elif each_section.endswith('-'):
                section_to_select_list.append([each_section_split[0], str(msa_in.get_alignment_length())])
            else:
                section_to_select_list.append(each_section_split)

    # check output folder
    if len(section_to_select_list) > 1:
        if os.path.isdir(op_dir) is True:
            if force_overwriting is True:
                os.system('rm -r %s' % op_dir)
            else:
                print('Output folder already exist, program exited!')
                exit()
        os.system('mkdir %s' % op_dir)

    # write out sections
    if len(section_to_select_list) == 1:
        current_section = msa_in[:, (int(section_to_select_list[0][0]) - 1):(int(section_to_select_list[0][1]))]
        if aln_out_ext == 'fasta':
            msa2fasta(current_section, op_dir)
        if aln_out_ext == 'phylip':
            msa2phylip(current_section, op_dir)
    else:
        for each_section in section_to_select_list:

            pwd_op_file     = '%s/%s.%s' % (op_dir, '-'.join(each_section), aln_out_ext)
            current_section = msa_in[:, (int(each_section[0])-1):(int(each_section[1]))]

            # write out
            if aln_out_ext == 'fasta':
                msa2fasta(current_section, pwd_op_file)
            if aln_out_ext == 'phylip':
                msa2phylip(current_section, pwd_op_file)

    print('MSA subset(s) exported to %s, Done!' % op_dir)


if __name__ == '__main__':

    # arguments for rename_seq_parser
    SliceMSA_parser = argparse.ArgumentParser(usage=SliceMSA_usage)
    SliceMSA_parser.add_argument('-i',      required=True,                         help='input MSA in fasta format')
    SliceMSA_parser.add_argument('-fi',     required=False, default='fasta',       help='format (NOT file extension) of input MSA, default: fasta')
    SliceMSA_parser.add_argument('-s',      required=True,                         help='columns to export, e.g. 200-300, -100, 50-')
    SliceMSA_parser.add_argument('-o',      required=True,                         help='output file or folder')
    SliceMSA_parser.add_argument('-fo',     required=False, default='fasta',       help='format of output MSA, select from fasta and phylip-relaxed, default: fasta')
    SliceMSA_parser.add_argument('-force',  required=False, action="store_true",   help='force overwrite existing output folder')
    args = vars(SliceMSA_parser.parse_args())
    SliceMSA(args)




import os
import glob
import argparse
from Bio import SeqIO


get_MAG_reads_long_usage = '''
======================================== get_MAG_reads_long example commands ========================================

BioSAK get_MAG_reads_long -r LongReads.fastq -s assembly.fasta.sam -m MAG_01.fa -o MAG_01_reads
BioSAK get_MAG_reads_long -r LongReads.fastq -s assembly.fasta.sam -m MAG_dir -x fa -o extracted_reads_dir -l 0
BioSAK get_MAG_reads_long -r LongReads.fastq -s assembly.fasta.sam -m MAG_dir -x fa -o extracted_reads_dir -l 5000

# Note
1. The input sam file is obtained by mapping quality-filtered reads to ALL (not a subset of) contigs derived 
   from the quality-filtered reads (e.g., flye produced assembly.fasta or medaka polished assemblies).
2. Important!!! The input Sam file and MAGs need to be generated from the same set of sequences (with the same id). 
3. If one read was mapped to multiple contigs from different MAGs, the read will be exported to the extracted 
   reads of all above MAGs.
4. If you use minimap2 for reads mapping, you might want to specify "--secondary=no" to minimise low-quality 
   secondary alignments:
   minimap2 -d assembly.fasta.mmi assembly.fasta
   minimap2 --secondary=no -t 6 -a assembly.fasta.mmi long_reads.fastq > assembly.fasta.sam

=====================================================================================================================
'''


def get_MAG_reads_long(args):

    reads_file          = args['r']
    sam_file            = args['s']
    mag_folder          = args['m']
    mag_ext             = args['x']
    min_read_len        = args['l']
    extracted_read_dir  = args['o']

    # create folder
    if os.path.isdir(extracted_read_dir) is True:
        print('Output folder detected, program exited!')
        print(extracted_read_dir)
        exit()

    mag_list = []
    if os.path.isfile(mag_folder) is True:
        mag_list = [mag_folder]
    elif os.path.isdir(mag_folder) is True:
        mag_re = '%s/*.%s' % (mag_folder, mag_ext)
        mag_list = glob.glob(mag_re)
        if len(mag_list) == 0:
            print('No MAG detected, program exited!')
            exit()
    else:
        print('No MAG file/dir detected, program exited!')
        exit()

    print('getting ctg_to_gnm dict')
    ctg_to_gnm_dict = {}
    ctgs_in_multi_mags_set = set()
    for each_gnm in mag_list:
        gnm_id = '.'.join(os.path.basename(each_gnm).strip().split('.')[:-1])
        for each_seq in SeqIO.parse(each_gnm, 'fasta'):
            ctg_id = each_seq.id
            if ctg_id not in ctg_to_gnm_dict:
                ctg_to_gnm_dict[ctg_id] = {gnm_id}
            else:
                ctg_to_gnm_dict[ctg_id].add(gnm_id)
                ctgs_in_multi_mags_set.add(ctg_id)

    # check if there is contig in multiple MAGs
    if len(ctgs_in_multi_mags_set) > 0:
        print('The following contigs found in multiple MAGs, proceed anyway!')
        print(','.join(sorted([i for i in ctgs_in_multi_mags_set])))

    print('getting read_to_gnm_dict')
    read_to_gnm_dict = {}
    for each_line in open(sam_file):
        if not each_line.startswith('@'):
            each_line_split = each_line.strip().split('\t')
            read_id = each_line_split[0]
            ref_id = each_line_split[2]
            ref_gnm_set = ctg_to_gnm_dict.get(ref_id, set())
            for each_ref_gnm in ref_gnm_set:
                if read_id not in read_to_gnm_dict:
                    read_to_gnm_dict[read_id] = {each_ref_gnm}
                else:
                    read_to_gnm_dict[read_id].add(each_ref_gnm)

    os.system('mkdir %s' % extracted_read_dir)

    print('Extracting reads')
    for each_read in SeqIO.parse(reads_file, 'fastq'):
        read_id = each_read.id
        read_len = len(each_read.seq)
        read_gnms = read_to_gnm_dict.get(read_id, {})
        if read_len >= min_read_len:
            if read_gnms != {}:
                for each_gnm in read_gnms:
                    pwd_read_file = '%s/%s_reads_min%sbp.fastq' % (extracted_read_dir, each_gnm, min_read_len)
                    pwd_read_file_handle = open(pwd_read_file, 'a')
                    SeqIO.write(each_read, pwd_read_file_handle, 'fastq')
                    pwd_read_file_handle.close()

    # print flye cmds
    print('Suggested flye commands:')
    for each_gnm in sorted(mag_list):
        gnm_id = '.'.join(os.path.basename(each_gnm).strip().split('.')[:-1])
        flye_cmd = 'flye --meta --threads 12 --out-dir flye_reassemble_wd_%s --nano-raw %s_reads_min%sbp.fastq' % (gnm_id, gnm_id, min_read_len)
        print(flye_cmd)

    print('Done!')


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-r',    required=True,                             help='reads file')
    parser.add_argument('-s',    required=True,                             help='sam file')
    parser.add_argument('-m',    required=True,                             help='MAG file/folder')
    parser.add_argument('-x',    required=False, default='fasta',           help='MAG extension, default: fasta')
    parser.add_argument('-l',    required=False, type=int, default=3000,    help='minimal read length (in bp), default: 3000')
    parser.add_argument('-o',    required=True,                             help='Output folder')
    args = vars(parser.parse_args())
    get_MAG_reads_long(args)



import argparse
from Bio import SeqIO


OneLineAln_usage = '''
========================= OneLineAln example commands =========================

BioSAK OneLineAln -in MarkerGenes.aln -out MarkerGenes_OneLine.aln
BioSAK OneLineAln -in MarkerGenes.aln -out MarkerGenes_OneLine.aln -upper

===============================================================================
'''

def OneLineAln(args):

    aln_in_fasta     = args['in']
    aln_out_one_line = args['out']
    to_uppercase     = args['upper']

    # get longest_seq_id
    longest_seq_id = 0
    for seq in SeqIO.parse(aln_in_fasta, 'fasta'):
        if len(seq.id) > longest_seq_id:
            longest_seq_id = len(seq.id)

    # write out in new format
    aln_in_one_line_handle = open(aln_out_one_line, 'w')
    for seq in SeqIO.parse(aln_in_fasta, 'fasta'):
        seq_id_polished = seq.id + (longest_seq_id - len(seq.id))*' '
        if to_uppercase is True:
            aln_in_one_line_handle.write('%s\t%s\n' % (seq_id_polished, str(seq.seq).upper()))
        else:
            aln_in_one_line_handle.write('%s\t%s\n' % (seq_id_polished, str(seq.seq)))

    aln_in_one_line_handle.close()


if __name__ == '__main__':

    OneLineAln_parser = argparse.ArgumentParser(usage=OneLineAln_usage)
    OneLineAln_parser.add_argument('-in',     required=True,                        help='input MSA in fasta format')
    OneLineAln_parser.add_argument('-out',    required=False, default=None,         help='output file')
    OneLineAln_parser.add_argument('-upper',  required=False, action='store_true',  help='turn to uppercase')
    args = vars(OneLineAln_parser.parse_args())
    OneLineAln(args)



import os
import argparse


koala_usage = '''
============= koala example commands =============

BioSAK koala -i user_ko.txt -o separated_ko_dir

==================================================
'''


def koala(args):

    file_in             = args['i']
    op_dir              = args['o']
    force_create_op_dir = args['f']

    # create op_dir
    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('output directory exist, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    # read in user_ko.txt
    gnm_to_annotation_dict = dict()
    for each in open(file_in):
        each_split = each.strip().split('\t')
        gnm_id = '_'.join(each_split[0].split('_')[:-1])
        if gnm_id not in gnm_to_annotation_dict:
            gnm_to_annotation_dict[gnm_id] = set()
        else:
            gnm_to_annotation_dict[gnm_id].add(each.strip())

    # write out
    for each_gnm in gnm_to_annotation_dict:
        pwd_annotation_file = '%s/%s.txt' % (op_dir, each_gnm)
        annotation_list = sorted(list(gnm_to_annotation_dict[each_gnm]))
        pwd_annotation_file_handle = open(pwd_annotation_file, 'w')
        pwd_annotation_file_handle.write('\n'.join(annotation_list))
        pwd_annotation_file_handle.close()

    print('Done!')


if __name__ == '__main__':

    koala_parser = argparse.ArgumentParser(usage=koala_usage)
    koala_parser.add_argument('-i', required=True,                          help='user_ko.txt from BlastKOALA or GhostKOALA')
    koala_parser.add_argument('-o', required=True,                          help='output directory')
    koala_parser.add_argument('-f', required=False, action="store_true",    help='force overwrite')
    args = vars(koala_parser.parse_args())
    koala(args)



import os
import glob
import argparse
from Bio import SeqIO


rename_seq_usage = '''
======================= rename_seqs.py example commands =======================

python3 rename_seqs.py -i gnm_dir -x fa -o op_dir

===============================================================================
'''

def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def rename_seqs(args):

    seq_file_in     = args['i']
    file_extension  = args['x']
    op_dir          = args['o']


    seq_in_re = '%s/*.%s' % (seq_file_in, file_extension)
    seq_in_list = glob.glob(seq_in_re)

    if len(seq_in_list) == 0:
        print('No sequence file detected, program exited!')
        exit()

    if os.path.isdir(op_dir) is True:
        print('Output folder detected, program exited: %s' % op_dir)
        exit()
    else:
        os.mkdir(op_dir)


    file_index = 1
    for each_file in seq_in_list:
        f_path, f_base, f_ext = sep_path_basename_ext(each_file)

        print('Processing %s/%s: %s' % (file_index, len(seq_in_list), f_base))

        file_out = '%s/%s%s' % (op_dir, f_base, f_ext)

        file_out_handle = open(file_out, 'w')
        seq_index = 1
        for each_seq in SeqIO.parse(each_file, 'fasta'):
            file_out_handle.write('>%s_%s\n' % (f_base, seq_index))
            file_out_handle.write('%s\n' % each_seq.seq)
            seq_index += 1
        file_out_handle.close()
        file_index += 1

    print('Done!')


if __name__ == '__main__':

    rename_seq_parser = argparse.ArgumentParser(usage=rename_seq_usage)
    rename_seq_parser.add_argument('-i',         required=True,                         help='input sequence folder')
    rename_seq_parser.add_argument('-x',         required=False, default='fasta',       help='file extension, default: fasta')
    rename_seq_parser.add_argument('-o',         required=True,                         help='output folder')
    args = vars(rename_seq_parser.parse_args())
    rename_seqs(args)



import os
import glob
import gzip
import argparse
from Bio import SeqIO


ribbon_usage = '''
============================= ribbon example commands =============================

# Dependencies: opd

# example commands
odp="/scratch/PI/ocessongwz/Software/odp/scripts/odp"
ribbon="/scratch/PI/ocessongwz/Software/odp/scripts/odp_rbh_to_ribbon"
BioSAK ribbon -plot_lgs -co chrom_order.txt -o op_dir -fa demo_files -pep demo_files -chrom demo_files -f -odp $odp -odp_rbh_to_ribbon $ribbon

Note:
1. Species names can't have '_' char
2. Reference: https://github.com/conchoecia/odp
3. Demo data: https://github.com/songweizhi/BioSAK/tree/master/demo_data/ribbon

===================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def gff2chrom(gff_in, chrom_out):

    # This function was modified based on the NCBIgff2chrom.py from https://github.com/conchoecia/odp
    # This program parses a NCBI GFF annotation and generates a .chrom file
    # see https://github.com/conchoecia/odp for the specification

    gzipped = False
    for thisend in [".gz", ".gzip", ".GZ", ".GZIP", ".gzipped", ".GZIPPED"]:
        if gff_in.endswith(thisend):
            gzipped = True

    if gzipped:
        handle = gzip.open(gff_in, 'rt')
    else:
        handle = open(gff_in, "r")

    prots = dict()
    for line in handle:
        line = line.strip()
        splitd = line.split("\t")
        if line and len(splitd) > 7 and splitd[2] == "CDS" and "protein_id=" in line:
            pid = [x for x in splitd[8].split(";") if x.startswith("protein_id=")][0].replace("protein_id=", "")
            scaf = splitd[0]
            strand = splitd[6]
            start = int(splitd[3])
            stop = int(splitd[3])
            if pid not in prots:
                prots[pid] = {"scaf": scaf, "strand": strand, "start": start, "stop": stop}
            else:
                if start < prots[pid]["start"]:
                    prots[pid]["start"] = start
                if stop > prots[pid]["stop"]:
                    prots[pid]["stop"] = stop
    handle.close()

    # write out .chrom file
    chrom_out_handle = open(chrom_out, 'w')
    for pid in prots:
        chrom_out_handle.write("{}\t{}\t{}\t{}\t{}\n".format(pid, prots[pid]["scaf"], prots[pid]["strand"], prots[pid]["start"], prots[pid]["stop"]))
    chrom_out_handle.close()


def ribbon(args):

    fa_in                   = args['fa']
    fa_x                    = args['fax']
    gff_in                  = args['gff']
    gff_x                   = args['gffx']
    gbk_in                  = args['gbk']
    gbk_x                   = args['gbkx']
    pep_in                  = args['pep']
    pep_x                   = args['pepx']
    chrom_in                = args['chrom']
    chrom_x                 = args['chromx']
    op_dir                  = args['o']
    species_order_txt       = args['so']
    chrom_order_txt         = args['co']
    thread_num              = args['t']
    force_overwrite         = args['f']
    minscafsize             = args['m']
    odp_exe                 = args['odp']
    odp_rbh_to_ribbon_exe   = args['odp_rbh_to_ribbon']
    plot_all                = args['plot_all']
    plot_lgs                = args['plot_lgs']

    ################################################# define file name #################################################

    wd_path = os.path.abspath(os.getcwd())

    get_macrosynteny_plot_wd = '%s/get_macrosynteny_plot' % op_dir
    get_ribbon_diagram_wd    = '%s/get_ribbon_diagram'    % op_dir
    config_yaml_macrosynteny = '%s/config.yaml'           % get_macrosynteny_plot_wd
    cmd_txt                  = '%s/commands.txt'          % op_dir

    ####################################################################################################################

    fa_f_re   = '%s/*.%s' % (fa_in, fa_x)
    fa_f_list = glob.glob(fa_f_re)

    species_to_chrom_dict = dict()
    for each_fa in fa_f_list:
        _, _, fa_base, _ = sep_path_basename_ext(each_fa)
        qualified_ctg_set = []
        for each_seq in SeqIO.parse(each_fa, 'fasta'):
            seq_len = len(each_seq.seq)
            if seq_len >= minscafsize:
                qualified_ctg_set.append(each_seq.id)
        species_to_chrom_dict[fa_base] = qualified_ctg_set

    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()

    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % get_macrosynteny_plot_wd)
    os.system('mkdir %s' % get_ribbon_diagram_wd)

    ####################################################################################################################

    species_order_list = []
    if species_order_txt is not None:
        for each_s in open(species_order_txt):
            species_order_list.append(each_s.strip())
    else:
        species_order_list = sorted(list(species_to_chrom_dict.keys()))

    chrom_order_dict = dict()
    if chrom_order_txt is not None:
        for each_c in open(chrom_order_txt):
            each_c_split = each_c.strip().split('\t')
            s_id = each_c_split[0]
            c_id = each_c_split[1]
            if s_id not in chrom_order_dict:
                chrom_order_dict[s_id] = []
            chrom_order_dict[s_id].append(c_id)

    fa_in_abspath                    = os.path.abspath(fa_in)
    op_dir_abspath                   = os.path.abspath(op_dir)
    get_macrosynteny_plot_wd_abspath = os.path.abspath(get_macrosynteny_plot_wd)
    get_ribbon_diagram_wd_abspath    = os.path.abspath(get_ribbon_diagram_wd)

    ################################### get config.yaml for getting macrosynteny plot ##################################

    # get config.yaml for getting macrosynteny plot
    config_yaml_macrosynteny_handle = open(config_yaml_macrosynteny, 'w')
    config_yaml_macrosynteny_handle.write('ignore_autobreaks: True\n')
    config_yaml_macrosynteny_handle.write('diamond_or_blastp: "diamond"\n')
    config_yaml_macrosynteny_handle.write('duplicate_proteins: "pass"\n')

    # write out plot_LGs
    if plot_lgs is True:
        config_yaml_macrosynteny_handle.write('plot_LGs: True\n')
    else:
        config_yaml_macrosynteny_handle.write('plot_LGs: False\n')

    config_yaml_macrosynteny_handle.write('plot_sp_sp: True\n')
    config_yaml_macrosynteny_handle.write('\n')

    # write out species section
    config_yaml_macrosynteny_handle.write('species:\n')
    for each_species in species_order_list:
        current_fa    = '%s/%s.fa'    % (fa_in_abspath, each_species)
        current_pep   = '%s/%s.pep'   % (fa_in_abspath, each_species)
        current_chrom = '%s/%s.chrom' % (fa_in_abspath, each_species)
        config_yaml_macrosynteny_handle.write('  %s:\n' % each_species)
        config_yaml_macrosynteny_handle.write('    proteins:    %s\n' % current_pep)
        config_yaml_macrosynteny_handle.write('    chrom:       %s\n' % current_chrom)
        config_yaml_macrosynteny_handle.write('    genome:      %s\n' % current_fa)
        config_yaml_macrosynteny_handle.write('    minscafsize: %s\n' % minscafsize)
    config_yaml_macrosynteny_handle.close()

    ############################################### get macrosynteny plot ##############################################

    snakemake_cmd_macrosynteny = 'snakemake --cores %s --snakefile %s' % (thread_num, odp_exe)

    # write out commands
    cmd_txt_handle = open(cmd_txt, 'w')
    cmd_txt_handle.write('cd %s\n' % get_macrosynteny_plot_wd_abspath)
    cmd_txt_handle.write(snakemake_cmd_macrosynteny + '\n')
    cmd_txt_handle.write('\n')
    cmd_txt_handle.close()

    # get macrosynteny plot
    print(snakemake_cmd_macrosynteny)
    os.chdir(get_macrosynteny_plot_wd_abspath)
    os.system(snakemake_cmd_macrosynteny)

    #################################### get config.yaml for getting ribbon diagram ####################################

    os.chdir(wd_path)
    step2_figures_abspath = '%s/odp/step2-figures'                % get_macrosynteny_plot_wd_abspath
    snakemake_cmd_ribbon  = 'snakemake --cores %s --snakefile %s' % (thread_num, odp_rbh_to_ribbon_exe)

    step2_figures_sub_dir_list = next(os.walk(step2_figures_abspath))[1]
    synteny_plot_list = ['synteny_nocolor']
    for each_sub_dir in step2_figures_sub_dir_list:
        if each_sub_dir.startswith('synteny_coloredby'):
            synteny_plot_list.append(each_sub_dir)

    for color_setting in synteny_plot_list:
        current_synteny_dir        = '%s/%s'          % (step2_figures_abspath, color_setting)
        current_wd                 = '%s/%s'          % (get_ribbon_diagram_wd_abspath, color_setting)
        current_config_yaml_ribbon = '%s/config.yaml' % current_wd
        os.mkdir(current_wd)

        with open(cmd_txt, 'a') as cmd_txt_handle:
            cmd_txt_handle.write('cd %s\n' % current_wd)
            cmd_txt_handle.write(snakemake_cmd_ribbon + '\n')
            cmd_txt_handle.write('\n')

        config_yaml_ribbon_handle = open(current_config_yaml_ribbon, 'w')
        config_yaml_ribbon_handle.write('chr_sort_order: optimal-chr-or\n')
        if plot_all is True:
            config_yaml_ribbon_handle.write('plot_all: True\n')
        else:
            config_yaml_ribbon_handle.write('plot_all: False\n')
        config_yaml_ribbon_handle.write('\n')

        # write out species_order section
        config_yaml_ribbon_handle.write('species_order:\n')
        for species in species_order_list:
            config_yaml_ribbon_handle.write('  - %s\n' % species)
        config_yaml_ribbon_handle.write('\n')

        # write out chromorder section
        config_yaml_ribbon_handle.write('chromorder:\n')
        if chrom_order_txt is not None:
            for each_s in chrom_order_dict:
                config_yaml_ribbon_handle.write('  %s:\n' % each_s)
                for each_chrom in chrom_order_dict[each_s]:
                    config_yaml_ribbon_handle.write('    - %s\n' % each_chrom)
            config_yaml_ribbon_handle.write('\n')
        else:
            for each_species in species_order_list:
                current_chrom_list = species_to_chrom_dict.get(each_species, [])
                if len(current_chrom_list) > 0:
                    config_yaml_ribbon_handle.write('  %s:\n' % each_species)
                    for each_chrom in current_chrom_list:
                        config_yaml_ribbon_handle.write('    - %s\n' % each_chrom)
                config_yaml_ribbon_handle.write('\n')

        # write out rbh_directory section
        config_yaml_ribbon_handle.write('rbh_directory: %s\n' % current_synteny_dir)
        config_yaml_ribbon_handle.write('\n')

        # write out species section
        config_yaml_ribbon_handle.write('species:\n')
        for each_species in species_order_list:
            current_fa    = '%s/%s.fa'    % (fa_in_abspath, each_species)
            current_pep   = '%s/%s.pep'   % (fa_in_abspath, each_species)
            current_chrom = '%s/%s.chrom' % (fa_in_abspath, each_species)
            config_yaml_ribbon_handle.write('  %s:\n' % each_species)
            config_yaml_ribbon_handle.write('    proteins:    %s\n' % current_pep)
            config_yaml_ribbon_handle.write('    chrom:       %s\n' % current_chrom)
            config_yaml_ribbon_handle.write('    genome:      %s\n' % current_fa)
            config_yaml_ribbon_handle.write('    minscafsize: %s\n' % minscafsize)
        config_yaml_ribbon_handle.close()

    ################################################## run snakemake ###################################################

    # get ribbon diagram
    for color_setting in synteny_plot_list:

        current_wd              = '%s/%s'           % (get_ribbon_diagram_wd_abspath, color_setting)
        pwd_plot_file           = '%s/output.pdf'   % current_wd
        pwd_plot_file_relocated = '%s/%s.pdf'       % (op_dir_abspath, color_setting.replace('synteny_', 'ribbon_'))

        os.chdir(current_wd)
        os.system(snakemake_cmd_ribbon)

        if os.path.isfile(pwd_plot_file) is True:
            os.system('mv %s %s' % (pwd_plot_file, pwd_plot_file_relocated))

    ################################################### final report ###################################################

    print('Ribbon diagram exported to %s' % op_dir)
    print('Done!')

    ####################################################################################################################


if __name__ == '__main__':

    ribbon_parser = argparse.ArgumentParser(usage=ribbon_usage)
    ribbon_parser.add_argument('-o',                    required=True,                                  help='output directory')
    ribbon_parser.add_argument('-fa',                   required=True,                                  help='fa file directory')
    ribbon_parser.add_argument('-fax',                  required=False, default='fa',                   help='fa file extension, default: fa')
    ribbon_parser.add_argument('-gff',                  required=False,                                 help='gff file directory')
    ribbon_parser.add_argument('-gffx',                 required=False, default='gff',                  help='gff file extension, default: gff')
    ribbon_parser.add_argument('-gbk',                  required=False,                                 help='gbk file directory')
    ribbon_parser.add_argument('-gbkx',                 required=False, default='gbk',                  help='gbk file extension, default: gbk')
    ribbon_parser.add_argument('-pep',                  required=False,                                 help='pep file directory')
    ribbon_parser.add_argument('-pepx',                 required=False, default='pep',                  help='pep file extension, default: pep')
    ribbon_parser.add_argument('-chrom',                required=False,                                 help='chrom file directory')
    ribbon_parser.add_argument('-chromx',               required=False, default='chrom',                help='chrom file extension, default: chrom')
    ribbon_parser.add_argument('-so',                   required=False, default=None,                   help='species order in the ribbon diagram')
    ribbon_parser.add_argument('-co',                   required=False, default=None,                   help='chromosome order in the ribbon diagram')
    ribbon_parser.add_argument('-m',                    required=False, type=int, default=10,           help='minscafsize, default: 10')
    ribbon_parser.add_argument('-t',                    required=False, type=int, default=1,            help='number of core, default: 1')
    ribbon_parser.add_argument('-f',                    required=False, action="store_true",            help='force overwrite')
    ribbon_parser.add_argument('-plot_all',             required=False, action="store_true",            help='plot_all')
    ribbon_parser.add_argument('-plot_lgs',             required=False, action="store_true",            help='plot_LGs')
    ribbon_parser.add_argument('-odp',                  required=False, default='odp',                  help='executable file odp, default: odp')
    ribbon_parser.add_argument('-odp_rbh_to_ribbon',    required=False, default='odp_rbh_to_ribbon',    help='executable file odp_rbh_to_ribbon, default: odp_rbh_to_ribbon')
    args = vars(ribbon_parser.parse_args())
    ribbon(args)



import argparse
import pandas as pd


transpose_usage = '''
============ transpose example commands ============

BioSAK transpose -i demo_df.txt -o demo_df_T.txt

# Input data matrix need to be tab separated

====================================================
'''


def transpose(args):

    file_in  = args['i']
    file_out = args['o']

    df = pd.read_csv(file_in, sep='\t', header=0, index_col=0)
    df_t = df.T
    df_t.to_csv(file_out, sep='\t')


if __name__ == '__main__':

    transpose_parser = argparse.ArgumentParser(usage=transpose_usage)
    transpose_parser.add_argument('-i',     required=True,                          help='input file')
    transpose_parser.add_argument('-o',     required=True,                          help='output file')
    args = vars(transpose_parser.parse_args())
    transpose(args)



import os
import glob
import argparse
from Bio import SeqIO
from Bio import AlignIO


ConvertMSA_usage = '''
================================= ConvertMSA example commands =================================

# phylip to fasta
BioSAK ConvertMSA -i concatenated.phy -fi phylip-relaxed -o concatenated.fasta -fo fasta
BioSAK ConvertMSA -i phy_files -fi phylip-relaxed -xi phy -o MSA_in_fasta -fo fasta -xo fa 

# examples of alignment format (https://biopython.org/wiki/AlignIO): 
fasta, phylip, phylip-relaxed, phylip-sequential, clustal

===============================================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def ConvertMSA(args):

    aln_in            = args['i']
    aln_in_ext        = args['xi']
    aln_in_format     = args['fi']
    aln_out           = args['o']
    aln_out_ext       = args['xo']
    aln_out_format    = args['fo']
    one_line          = args['oneline']
    no_gap            = args['nogap']
    force_overwriting = args['f']

    if ((one_line is True) and (aln_out_format != 'fasta')) or ((no_gap is True) and (aln_out_format != 'fasta')):
        print('Please provide "-oneline" and/or "-nogap" only if "-fo" is fasta')
        exit()

    if os.path.isfile(aln_in) is True:
        if (one_line is False) and (no_gap is False):
            AlignIO.convert(aln_in, aln_in_format, aln_out, aln_out_format)
        else:
            aln_out_tmp = aln_out + '.tmp'
            AlignIO.convert(aln_in, aln_in_format, aln_out_tmp, aln_out_format)
            pwd_aln_out_handle = open(aln_out, 'w')
            for each_seq in SeqIO.parse(aln_out_tmp, 'fasta'):
                seq_id = each_seq.id
                seq_sequence = str(each_seq.seq)
                if no_gap is False:
                    pwd_aln_out_handle.write('>%s\n' % seq_id)
                    pwd_aln_out_handle.write('%s\n' % seq_sequence)
                else:
                    pwd_aln_out_handle.write('>%s\n' % seq_id)
                    pwd_aln_out_handle.write('%s\n' % seq_sequence.replace('-', ''))
            pwd_aln_out_handle.close()
            os.system('rm %s' % aln_out_tmp)

        print('Done!')

    elif os.path.isdir(aln_in) is True:
        aln_in_re = '%s/*.%s' % (aln_in, aln_in_ext)
        aln_in_list = [os.path.basename(file_name) for file_name in glob.glob(aln_in_re)]

        # check input
        if len(aln_in_list) == 0:
            print('Input file not detected, program exited!')
            exit()

        # check output folder
        if os.path.isdir(aln_out) is True:
            if force_overwriting is True:
                os.system('rm -r %s' % aln_out)
            else:
                print('Output folder already exist, program exited!')
                exit()
        os.system('mkdir %s' % aln_out)

        # convert
        for each_aln_in in aln_in_list:

            aln_in_path, aln_in_basename, aln_in_ext = sep_path_basename_ext(each_aln_in)
            pwd_aln_in      = '%s/%s'        % (aln_in, each_aln_in)
            pwd_aln_out     = '%s/%s.%s'     % (aln_out, aln_in_basename, aln_out_ext)
            pwd_aln_out_tmp = '%s/%s_tmp.%s' % (aln_out, aln_in_basename, aln_out_ext)

            if (one_line is False) and (no_gap is False):
                AlignIO.convert(pwd_aln_in, aln_in_format, pwd_aln_out, aln_out_format)
            else:
                AlignIO.convert(pwd_aln_in, aln_in_format, pwd_aln_out_tmp, aln_out_format)
                pwd_aln_out_handle = open(pwd_aln_out, 'w')
                for each_seq in SeqIO.parse(pwd_aln_out_tmp, 'fasta'):
                    seq_id = each_seq.id
                    seq_sequence = str(each_seq.seq)
                    if no_gap is False:
                        pwd_aln_out_handle.write('>%s\n' % seq_id)
                        pwd_aln_out_handle.write('%s\n' % seq_sequence)
                    else:
                        sequence_no_gap = seq_sequence.replace('-', '')
                        if len(sequence_no_gap) > 0:
                            pwd_aln_out_handle.write('>%s\n' % seq_id)
                            pwd_aln_out_handle.write('%s\n' % sequence_no_gap)
                pwd_aln_out_handle.close()
                os.system('rm %s' % pwd_aln_out_tmp)
        print('Done!')
    else:
        print('Input file not found, program exited!')


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser(usage=ConvertMSA_usage)
    parser.add_argument('-i',       required=True,                       help='input alignment')
    parser.add_argument('-xi',      required=False, default='aln',       help='input alignment extension')
    parser.add_argument('-fi',      required=True,                       help='input alignment format, e.g., fasta, phylip')
    parser.add_argument('-o',       required=True,                       help='output alignment')
    parser.add_argument('-xo',      required=False, default='aln',       help='output alignment extension')
    parser.add_argument('-fo',      required=True,                       help='output alignment format, e.g., fasta, phylip')
    parser.add_argument('-oneline', required=False, action="store_true", help='put sequence in single line, available if -fo is fasta')
    parser.add_argument('-nogap',   required=False, action="store_true", help='remove gaps from alignment, available if -fo is fasta')
    parser.add_argument('-f',       required=False, action="store_true", help='force overwrite existing output folder')
    args = vars(parser.parse_args())
    ConvertMSA(args)



import os
import argparse


fq2fa_usage = '''
========== fq2fa example commands ==========

BioSAK fq2fa -i reads_R1.fq -o reads_R1.fa
BioSAK fq2fa -i reads_R2.fq -o reads_R2.fa

============================================
'''


def fq2fa(args):
    fq_in   = args['i']
    fa_out  = args['o']
    sed_cmd = "sed -n '1~4s/^@/>/p;2~4p' %s > %s" % (fq_in, fa_out)
    os.system(sed_cmd)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', required=True, help='input fastq')
    parser.add_argument('-o', required=True, help='output fasta')
    args = vars(parser.parse_args())
    fq2fa(args)



import argparse
from scipy.stats import mannwhitneyu


mannwhitneyu_usage = '''
============== mannwhitneyu example commands ============== 

BioSAK mannwhitneyu -1 treatment_1.txt -2 treatment_2.txt

===========================================================
'''


def mannwhitneyu(args):
    s1_txt = args['1']
    s2_txt = args['2']

    s1_list = []
    for e1 in open(s1_txt):
        s1_list.append(float(e1.strip()))

    s2_list = []
    for e2 in open(s2_txt):
        s2_list.append(float(e2.strip()))

    _, p_value = mannwhitneyu(s1_list, s2_list)

    print('P value: %s' % p_value)


if __name__ == '__main__':

    mannwhitneyu_parser = argparse.ArgumentParser()
    mannwhitneyu_parser.add_argument('-1', required=True, help='sample 1')
    mannwhitneyu_parser.add_argument('-2', required=True, help='sample 2')
    args = vars(mannwhitneyu_parser.parse_args())
    mannwhitneyu(args)



import argparse
import numpy as np


MeanMappingDepth_usage = '''
========== MeanMappingDepth example commands ==========

BioSAK MeanMappingDepth -depth ctgs.bam.depth
BioSAK MeanMappingDepth -depth ctgs.bam.depth -T

# get depth file with samtools
samtools depth ctgs.bam > ctgs.bam.depth

=======================================================
'''


def MeanMappingDepth(args):

    sam_depth  = args['depth']
    total_mean = args['T']

    # get length and total depth
    seq_to_len_and_depth_dict = {}
    for each_line in open(sam_depth):
        each_line_split = each_line.strip().split('\t')
        seq_id = each_line_split[0]
        bp_depth = int(each_line_split[2])
        if seq_id not in seq_to_len_and_depth_dict:
            seq_to_len_and_depth_dict[seq_id] = [1, bp_depth]
        else:
            seq_to_len_and_depth_dict[seq_id][0] += 1
            seq_to_len_and_depth_dict[seq_id][1] += bp_depth

    # get mean depth
    all_seq_len   = 0
    all_seq_depth = 0
    all_seq_mean_depth_list = []
    if total_mean is False:
        print('Seq\tLength(bp)\tMeanDepth')
    for each_seq in seq_to_len_and_depth_dict:
        current_seq_len   = seq_to_len_and_depth_dict[each_seq][0]
        current_seq_depth = seq_to_len_and_depth_dict[each_seq][1]
        all_seq_len += current_seq_len
        all_seq_depth += current_seq_depth
        seq_mean_depth = float("{0:.2f}".format(current_seq_depth/current_seq_len))
        if total_mean is False:
            print('%s\t%s\t%s' % (each_seq, seq_to_len_and_depth_dict[each_seq][0], seq_mean_depth))

        all_seq_mean_depth_list.append(seq_mean_depth)

    if total_mean is True:
        all_seq_mean_depth = float("{0:.2f}".format(all_seq_depth/all_seq_len))

        # report
        print('Sequence num  = %s'      % len(seq_to_len_and_depth_dict))
        print('Total length  = %s bp'   % all_seq_len)
        print('Highest depth = %s'      % max(all_seq_mean_depth_list))
        print('Lowest depth  = %s'      % min(all_seq_mean_depth_list))
        print('Median depth  = %s'      % np.median(all_seq_mean_depth_list))
        print('Mean depth    = %s'      % all_seq_mean_depth)


if __name__ == '__main__':

    MeanMappingDepth_parser = argparse.ArgumentParser(usage=MeanMappingDepth_usage)

    MeanMappingDepth_parser.add_argument('-depth', required=True,                       help='input depth file from "samtools depth" ')
    MeanMappingDepth_parser.add_argument('-T',     required=False, action="store_true", help='get overall stats')

    args = vars(MeanMappingDepth_parser.parse_args())

    MeanMappingDepth(args)



#!/usr/bin/env python3

import os
import random
import argparse
from Bio import SeqIO
from Bio.Seq import Seq


Reads_simulator_usage = '''
===================== Reads_simulator example commands =====================

# simulate 100000 pairs of reads from ref.fa
BioSAK Reads_simulator -p Test -r ref.fa -l 250 -i 300 -split -n 100000

# simulate reads to a depth of 50X
BioSAK Reads_simulator -p Test -r ref.fa -l 250 -i 300 -split -d 50

============================================================================
'''


def Reads_simulator(args):

    op_prefix       = args['p']
    pwd_genome_file = args['r']
    read_number     = args['n']
    read_depth      = args['d']
    read_length     = args['l']
    insert_size     = args['i']
    split           = args['split']

    if (read_number is None) and (read_depth is None):
        print('Please either provide the number of read pairs to simulate (-n) or the desired sequencing depth (-d).')
        print('Program exited!')
        exit()
    elif (read_number is not None) and (read_depth is not None):
        print('"-d" and "-n" can not be specified at the same time ')
        print('Program exited!')
        exit()

    # read in reference sequences
    ref_seq_total_len = 0
    ref_seq_dict = {}
    ref_seq_len_dict = {}
    for each_seq in SeqIO.parse(pwd_genome_file, 'fasta'):
        seq_id = each_seq.id
        seq_str = str(each_seq.seq)
        ref_seq_dict[seq_id] = seq_str
        ref_seq_len_dict[seq_id] = len(seq_str)
        ref_seq_total_len += len(seq_str)

    # calculate the number of reads to simulate
    if (read_number is None) and (read_depth is not None):
        read_number = round(ref_seq_total_len*read_depth/(read_length*2))
        print('The number of read pairs to simulate: %s' % read_number)

    # get the number of reads to simulate from each ref seq
    ref_to_read_num_dict = {}
    current_ref_index = 1
    total_assigned_read_num = 0
    for each_ref_seq in ref_seq_len_dict:
        if current_ref_index == len(ref_seq_len_dict):
            current_ref_seq_read_num = read_number - total_assigned_read_num
        else:
            current_ref_seq_len_pct = ref_seq_len_dict[each_ref_seq]/ref_seq_total_len
            current_ref_seq_read_num = round(current_ref_seq_len_pct*read_number)
            total_assigned_read_num += current_ref_seq_read_num
        ref_to_read_num_dict[each_ref_seq] = current_ref_seq_read_num
        current_ref_index += 1

    # simulate reads
    output_r1_handle = ''
    output_r2_handle = ''
    output_combined_handle = ''
    if split == 1:
        output_r1 = '%s_R1.fasta' % (op_prefix)
        output_r2 = '%s_R2.fasta' % (op_prefix)
        output_r1_handle = open(output_r1, 'w')
        output_r2_handle = open(output_r2, 'w')
    else:
        output_combined = '%s_R12.fasta' % (op_prefix)
        output_combined_handle = open(output_combined, 'w')

    overall_n = 1
    for each_ref_seq in ref_to_read_num_dict:

        ref_seq_str = ref_seq_dict[each_ref_seq]
        ref_seq_read_num = ref_to_read_num_dict[each_ref_seq]

        sequence_length = len(ref_seq_str)
        fragment_length = 2 * read_length + insert_size

        n = 1
        while n <= ref_seq_read_num:
            rdm_num = random.randint(1, sequence_length)
            current_fragment = ''
            if (rdm_num + fragment_length) <= sequence_length:
                current_fragment = ref_seq_str[rdm_num - 1: rdm_num + fragment_length - 1]
            elif (rdm_num + fragment_length) > sequence_length:
                seq_part_1_seq = ref_seq_str[rdm_num - 1:]
                seq_part_2_seq = ref_seq_str[:fragment_length - sequence_length + rdm_num - 1]
                current_fragment = seq_part_1_seq + seq_part_2_seq
            current_fragment_r1 = current_fragment[:read_length]
            current_fragment_r2 = current_fragment[-read_length:]
            current_fragment_r2_reverse_complement = str(Seq(current_fragment_r2).reverse_complement())
            current_read_r1_id = '%s_%s.1' % (op_prefix, overall_n)
            current_read_r2_id = '%s_%s.2' % (op_prefix, overall_n)

            if split == 1:
                output_r1_handle.write('>%s\n' % current_read_r1_id)
                output_r1_handle.write('%s\n'  % current_fragment_r1)
                output_r2_handle.write('>%s\n' % current_read_r2_id)
                output_r2_handle.write('%s\n'  % current_fragment_r2_reverse_complement)
            else:
                output_combined_handle.write('>%s\n' % current_read_r1_id)
                output_combined_handle.write('%s\n'  % current_fragment_r1)
                output_combined_handle.write('>%s\n' % current_read_r2_id)
                output_combined_handle.write('%s\n'  % current_fragment_r2_reverse_complement)

            n += 1
            overall_n += 1

    if split == 1:
        output_r1_handle.close()
        output_r2_handle.close()
        print('Simulated reads exported to %s and %s' % (output_r1, output_r2))
    else:
        output_combined_handle.close()
        print('Simulated reads exported to %s' % output_combined)

    print('Done!')


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-p',     required=True,                            help='output prefix')
    parser.add_argument('-r',     required=True,                            help='reference genome')
    parser.add_argument('-n',     required=False, type=int,   default=None, help='number of read pairs to simulate')
    parser.add_argument('-d',     required=False, type=float, default=None, help='desired read depth (X)')
    parser.add_argument('-l',     required=True,  type=int,                 help='read length (bp)')
    parser.add_argument('-i',     required=True,  type=int,                 help='insert size (bp)')
    parser.add_argument('-split', action="store_true",                      help='Export forward and reverse reads to separate files')
    args = vars(parser.parse_args())
    Reads_simulator(args)



import os
import glob
import shutil
from Bio import SeqIO


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


# define input and output files
faa_file_folder = '/Users/songweizhi/Desktop/test'
output_folder = '/Users/songweizhi/Desktop/test_aa_comp'


force_create_folder(output_folder)

amino_acid_str = 'ARNDBCEQZGHILKMFPSTWYV'
amino_acid_list = [i for i in amino_acid_str]
faa_file_re = '%s/*.faa' % faa_file_folder
faa_file_list = [os.path.basename(file_name) for file_name in glob.glob(faa_file_re)]

for faa_file in faa_file_list:

    pwd_faa = '%s/%s' % (faa_file_folder, faa_file)
    faa_file_path, faa_file_basename, faa_file_extension = sep_path_basename_ext(pwd_faa)

    output_file = '%s/%s_aa_comp.txt' % (output_folder, faa_file_basename)
    output_file_handle = open(output_file, 'w')
    output_file_handle.write('Gene\t%s\n' % '\t'.join(amino_acid_list))
    for seq_record in SeqIO.parse(pwd_faa, 'fasta'):
        seq_record_id = seq_record.id
        seq_record_seq = str(seq_record.seq)

        aa_count_list = []
        for aa in amino_acid_str:
            aa_percent = seq_record_seq.count(aa)*100/len(seq_record_seq)
            aa_percent = float("{0:.2f}".format(aa_percent))
            aa_count_list.append(aa_percent)

        aa_count_list_str = [str(i) for i in aa_count_list]
        output_file_handle.write('%s\t%s\n' % (seq_record_id, '\t'.join(aa_count_list_str)))

    output_file_handle.close()




import argparse


def get_best_hit_all(file_in, file_out, taxonomy_dict):

    highest_bit_score_dict = {}
    for blast_hit in open(file_in):
        blast_hit_split = blast_hit.strip().split('\t')
        query_id = blast_hit_split[0]
        bit_score = float(blast_hit_split[12])

        if query_id not in highest_bit_score_dict:
            highest_bit_score_dict[query_id] = bit_score
        else:
            if bit_score > highest_bit_score_dict[query_id]:
                highest_bit_score_dict[query_id] = bit_score

    file_out_handle = open(file_out, 'w')
    file_out_handle.write('X.OTU.ID\tRef.ID\tIdentity\tAlignment_Length\tEvalue\tBit_Score\ttaxonomy\n')
    for blast_hit in open(file_in):
        blast_hit_split = blast_hit.strip().split('\t')
        query_id = blast_hit_split[0]
        ref_id = blast_hit_split[1]
        identity = float(blast_hit_split[2])
        Alignment_Length = int(blast_hit_split[3])
        evalue = float(blast_hit_split[10])
        bit_score = float(blast_hit_split[12])

        if bit_score == highest_bit_score_dict[query_id]:
            file_out_handle.write('%s\t%s\t%s\t%s\t%s\t%s\t%s\n' % (query_id, ref_id, identity, Alignment_Length, evalue, bit_score, taxonomy_dict[ref_id]))

    file_out_handle.close()


def get_best_hit_first(file_in, file_out, taxonomy_dict):

    file_out_handle = open(file_out, 'w')
    file_out_handle.write('X.OTU.ID\tRef.ID\tIdentity\tAlignment_Length\tEvalue\tBit_Score\ttaxonomy\n')
    best_hit_line = ''
    best_hit_query_id = ''
    best_hit_score = 0
    for blast_hit in open(file_in):
        blast_hit_split = blast_hit.strip().split('\t')
        query_id = blast_hit_split[0]
        bit_score = int(blast_hit_split[12])

        if best_hit_query_id == '':
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

        elif (query_id == best_hit_query_id) and (bit_score > best_hit_score):
            best_hit_score = bit_score
            best_hit_line = blast_hit

        elif query_id != best_hit_query_id:
            best_hit_line_split = best_hit_line.strip().split('\t')
            file_out_handle.write('%s\t%s\t%s\t%s\t%s\t%s\t%s\n' % (best_hit_line_split[0], best_hit_line_split[1], best_hit_line_split[2], best_hit_line_split[3], best_hit_line_split[10], float(best_hit_line_split[12]), taxonomy_dict[best_hit_line_split[1]]))
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

    best_hit_line_split = best_hit_line.strip().split('\t')
    file_out_handle.write('%s\t%s\t%s\t%s\t%s\t%s\t%s\n' % (best_hit_line_split[0], best_hit_line_split[1], best_hit_line_split[2], best_hit_line_split[3],best_hit_line_split[10], float(best_hit_line_split[12]), taxonomy_dict[best_hit_line_split[1]]))
    file_out_handle.close()


def get_best_hit(args):

    file_in =       args['i']
    file_out =      args['o']
    taxonomy_file = args['t']
    keep_all =      args['all']

    # store taxonomy info in dict
    taxonomy_dict = {}
    for ref in open(taxonomy_file):
        ref_split = ref.strip().split('\t')
        ref_id = ref_split[0]
        ref_taxon = ref_split[1]
        ref_taxon_split_no_rank = [i.split(':')[1] for i in ref_taxon.split(';')[:-1]]
        ref_taxon_no_rank = ';'.join(ref_taxon_split_no_rank[::-1])
        taxonomy_dict[ref_id] = ref_taxon_no_rank

    if keep_all is True:
        get_best_hit_all(file_in, file_out, taxonomy_dict)
    else:
        get_best_hit_first(file_in, file_out, taxonomy_dict)


if __name__ == '__main__':

    get_best_hit_parser = argparse.ArgumentParser()

    # arguments for COG_parser
    get_best_hit_parser.add_argument('-i',   required=True,                          help='blast results from BLCA')
    get_best_hit_parser.add_argument('-o',   required=True,                          help='output file')
    get_best_hit_parser.add_argument('-t',   required=True,                          help='taxonomy file')
    get_best_hit_parser.add_argument('-all', required=False, action='store_true',    help='keep all blast hits with highest bit score')

    args = vars(get_best_hit_parser.parse_args())

    get_best_hit(args)




import os
import glob
import shutil
import argparse
from Bio import SeqIO
import multiprocessing as mp
from datetime import datetime
from Bio.SeqRecord import SeqRecord


arCOG_parser_usage = '''
====================================== arCOG example commands ======================================

# Dependencies: blast+, diamond

# annotate protein sequences
BioSAK arCOG -m P -t 6 -db_dir arCOG18_db_dir -i genes.faa 
BioSAK arCOG -m P -t 6 -db_dir arCOG18_db_dir -i faa_files -x faa -d depth_files

# annotate DNA sequences (ORFs)
BioSAK arCOG -m N -t 6 -db_dir arCOG18_db_dir -i genes.ffn -d gene.depth
BioSAK arCOG -m N -t 6 -db_dir arCOG18_db_dir -i ffn_files -x ffn

# Prepare DB files:
cd path/to/your/arCOG_db_dir
wget https://ftp.ncbi.nih.gov/pub/wolf/COGs/arCOG/tmp.ar18/ar18.fa.gz
wget https://ftp.ncbi.nih.gov/pub/wolf/COGs/arCOG/tmp.ar18/ar18.ar14.02.csv
wget https://ftp.ncbi.nih.gov/pub/wolf/COGs/arCOG/tmp.ar18/arCOGdef.tab
wget https://ftp.ncbi.nih.gov/pub/COG/COG2020/data/fun-20.tab
gunzip ar18.fa.gz
makeblastdb -in ar18.fa -dbtype prot -parse_seqids -logfile ar18.fa.log
diamond makedb --in ar18.fa --db ar18.fa.dmnd --quiet

# For how it works, please refers to the help information of the COG2020 module (BioSAK COG2020 -h)

====================================================================================================
'''

time_format = '[%Y-%m-%d %H:%M:%S] '


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)

    os.mkdir(folder_to_create)


def sep_path_basename_ext(file_in):

    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'
    file_basename, file_ext = os.path.splitext(file_name)

    return file_path, file_basename, file_ext


def AnnotateNorm(file_in, skip_header, value_column, Divisor_value, file_out, file_out_header):

    file_out_handle = open(file_out, 'w')
    file_out_handle.write(file_out_header)
    line_num = 0
    for each_line in open(file_in):
        each_line_split = each_line.strip().split('\t')
        value_str = each_line_split[value_column - 1]
        if (skip_header is True and line_num > 0) or (skip_header is False):
            value_pct = 0
            if Divisor_value != 0:
                value_pct = float(value_str) * 100 / Divisor_value
            each_line_split[value_column - 1] = str(float("{0:.2f}".format(value_pct)))
            file_out_handle.write('%s\n' % '\t'.join(each_line_split))
        line_num += 1
    file_out_handle.close()


def get_gene_list_depth(gene_list, gene_to_depth_dict):

    total_depth = 0
    for gene in gene_list:
        gene_depth = gene_to_depth_dict[gene]
        total_depth += gene_depth

    return total_depth


def dna2aa(dna_file, aa_file):
    query_aa_handle = open(aa_file, 'w')
    for each in SeqIO.parse(dna_file, 'fasta'):
        each_aa = each.seq.translate()
        each_aa_record = SeqRecord(each_aa)
        each_aa_record.id = each.id
        each_aa_record.description = each.description
        SeqIO.write(each_aa_record, query_aa_handle, 'fasta')
    query_aa_handle.close()


def best_hit(args):

    file_in = args['i']
    file_out = args['o']

    file_out_handle = open(file_out, 'w')
    best_hit_line = ''
    best_hit_query_id = ''
    best_hit_score = 0
    for blast_hit in open(file_in):
        blast_hit_split = blast_hit.strip().split('\t')
        query_id = blast_hit_split[0]
        bit_score = float(blast_hit_split[11])

        if best_hit_query_id == '':
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score
        elif (query_id == best_hit_query_id) and (bit_score > best_hit_score):
            best_hit_score = bit_score
            best_hit_line = blast_hit
        elif query_id != best_hit_query_id:
            file_out_handle.write(best_hit_line)
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

    file_out_handle.write(best_hit_line)
    file_out_handle.close()


def arCOG_worker(argument_list):

    pwd_input_file =                    argument_list[0]
    pwd_prot2003_2014 =                 argument_list[1]
    protein_id_to_cog_id_dict =         argument_list[2]
    cog_id_to_category_dict =           argument_list[3]
    cog_id_to_description_dict =        argument_list[4]
    cog_category_list =                 argument_list[5]
    cog_category_to_description_dict =  argument_list[6]
    sequence_type =                     argument_list[7]
    output_folder =                     argument_list[8]
    thread_num =                        argument_list[9]
    run_diamond =                       argument_list[10]
    evalue_cutoff =                     argument_list[11]
    depth_file =                        argument_list[12]
    pct_by_all =                        argument_list[13]

    input_seq_no_path, input_seq_no_ext, input_seq_ext = sep_path_basename_ext(pwd_input_file)
    current_output_folder = '%s/%s_arCOG_wd' % (output_folder, input_seq_no_ext)

    pwd_blastp_output =                 '%s/%s_blastp.tab'                          % (current_output_folder, input_seq_no_ext)
    pwd_blastp_output_besthits =        '%s/%s_blastp_besthits.tab'                 % (current_output_folder, input_seq_no_ext)
    pwd_query_to_cog_txt =              '%s/%s_query_to_cog.txt'                    % (current_output_folder, input_seq_no_ext)
    pwd_cog_stats_copy =                '%s/%s_arcog_stats.txt'                     % (current_output_folder, input_seq_no_ext)
    pwd_cog_stats_depth =               '%s/%s_arcog_stats_depth.txt'               % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_copy =               '%s/%s_func_stats.txt'                      % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_depth =              '%s/%s_func_stats_depth.txt'                % (current_output_folder, input_seq_no_ext)
    pwd_cog_stats_copy_pct =            '%s/%s_arcog_stats_pct.txt'                 % (current_output_folder, input_seq_no_ext)
    pwd_cog_stats_depth_pct =           '%s/%s_arcog_stats_depth_pct.txt'           % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_copy_pct =           '%s/%s_func_stats_pct.txt'                  % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_depth_pct =          '%s/%s_func_stats_depth_pct.txt'            % (current_output_folder, input_seq_no_ext)
    pwd_cog_stats_copy_pct_by_all =     '%s/%s_arcog_stats_pct_by_all.txt'          % (current_output_folder, input_seq_no_ext)
    pwd_cog_stats_depth_pct_by_all =    '%s/%s_arcog_stats_depth_pct_by_all.txt'    % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_copy_pct_by_all =    '%s/%s_func_stats_pct_by_all.txt'           % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_depth_pct_by_all =   '%s/%s_func_stats_depth_pct_by_all.txt'     % (current_output_folder, input_seq_no_ext)

    force_create_folder(current_output_folder)

    input_seq_aa = ''
    if sequence_type in ['N', 'n']:
        input_seq_aa = '%s_aa.fasta' % input_seq_no_ext
        dna2aa(pwd_input_file, input_seq_aa)
    elif sequence_type in ['P', 'p']:
        input_seq_aa = pwd_input_file
    else:
        print('Specified input sequence type unrecognizable, program exited!')
        exit()

    # run blastp
    if run_diamond is False:
        os.system('blastp -query %s -db %s -out %s -evalue %s -outfmt 6 -show_gis -num_threads %s' % (input_seq_aa, pwd_prot2003_2014, pwd_blastp_output, evalue_cutoff, thread_num))
    else:
        os.system('diamond blastp -q %s --db %s.dmnd --out %s --evalue %s --outfmt 6 --threads %s --quiet' % (input_seq_aa, pwd_prot2003_2014, pwd_blastp_output, evalue_cutoff, thread_num))

    # keep only best hits
    best_hit({'i': pwd_blastp_output, 'o': pwd_blastp_output_besthits})

    # get query_to_ref_protein_dict
    query_to_ref_protein_dict = {}
    for each_hit in open(pwd_blastp_output_besthits):
        each_hit_split = each_hit.strip().split('\t')
        each_hit_query = each_hit_split[0]
        each_hit_subject = each_hit_split[1]
        each_hit_subject_no_dot = '_'.join(each_hit_subject.split('.'))
        query_to_ref_protein_dict[each_hit_query] = each_hit_subject_no_dot

    # get query sequences list
    query_seq_list = []
    for query_seq in SeqIO.parse(pwd_input_file, 'fasta'):
        query_seq_list.append(query_seq.id)

    # export annotation
    cog_id_num_dict = {}
    cog_cate_num_dict = {}
    cog_id_to_gene_member_dict = {}
    cog_cate_to_gene_member_dict = {}
    genes_with_cog = set()
    pwd_query_to_cog_txt_handle = open(pwd_query_to_cog_txt, 'w')
    pwd_query_to_cog_txt_handle.write('Query\tarCOG\tCategory\tDescription\n')
    for query_gene in sorted(query_seq_list):

        if query_gene not in query_to_ref_protein_dict:
            pwd_query_to_cog_txt_handle.write('%s\n' % (query_gene))

        else:
            db_protein_id = query_to_ref_protein_dict[query_gene]
            if db_protein_id not in protein_id_to_cog_id_dict:
                pwd_query_to_cog_txt_handle.write('%s\n' % (query_gene))

            else:
                cog_id_list = protein_id_to_cog_id_dict[db_protein_id]
                for cog_id in cog_id_list:
                    cog_cate = cog_id_to_category_dict.get(cog_id, 'na')
                    cog_des = cog_id_to_description_dict[cog_id]
                    pwd_query_to_cog_txt_handle.write('%s\t%s\t%s\t%s\n' % (query_gene, cog_id, ''.join(cog_cate), cog_des))
                    genes_with_cog.add(query_gene)

                    # update cog_id_num_dict
                    if cog_id not in cog_id_num_dict:
                        cog_id_num_dict[cog_id] = 1
                        cog_id_to_gene_member_dict[cog_id] = [query_gene]
                    else:
                        cog_id_num_dict[cog_id] += 1
                        cog_id_to_gene_member_dict[cog_id].append(query_gene)

                    # update cog_cate_num_dict
                    for each_cog_cate in cog_cate:
                        if each_cog_cate not in cog_cate_num_dict:
                            cog_cate_num_dict[each_cog_cate] = 1
                            cog_cate_to_gene_member_dict[each_cog_cate] = [query_gene]
                        else:
                            cog_cate_num_dict[each_cog_cate] += 1
                            cog_cate_to_gene_member_dict[each_cog_cate].append(query_gene)
    pwd_query_to_cog_txt_handle.close()

    # read in depth info
    gene_depth_dict = {}
    if depth_file is not None:
        for each_depth in open(depth_file):
            each_depth_split = each_depth.strip().split('\t')
            gene_depth_dict[each_depth_split[0]] = float(each_depth_split[1])

    # get depth of all query genes or genes with cog assignment
    if depth_file is not None:
        genes_with_cog_depth = get_gene_list_depth(genes_with_cog, gene_depth_dict)
        total_depth_for_all_query_genes = get_gene_list_depth(query_seq_list, gene_depth_dict)

    #################### export cog_stats_copy ####################

    pwd_cog_stats_copy_handle = open(pwd_cog_stats_copy, 'w')
    pwd_cog_stats_copy_handle.write('arCOG\tcopy\tDescription\n')
    for each_cog_id in cog_id_num_dict:
        each_cog_id_copy = cog_id_num_dict[each_cog_id]
        pwd_cog_stats_copy_handle.write('%s\t%s\t%s\n' % (each_cog_id, each_cog_id_copy, cog_id_to_description_dict[each_cog_id]))
    pwd_cog_stats_copy_handle.close()

    #################### export cog_stats_depth ####################

    if depth_file is not None:
        pwd_cog_stats_depth_handle = open(pwd_cog_stats_depth, 'w')
        pwd_cog_stats_depth_handle.write('COG\tdepth\tDescription\n')
        for each_cog_id in cog_id_to_gene_member_dict:
            each_cog_id_gene_member = cog_id_to_gene_member_dict[each_cog_id]
            each_cog_id_depth = 0
            for each_gene in each_cog_id_gene_member:
                each_gene_depth = gene_depth_dict[each_gene]
                each_cog_id_depth += each_gene_depth
            each_cog_id_depth = float("{0:.2f}".format(each_cog_id_depth))
            pwd_cog_stats_depth_handle.write('%s\t%s\t%s\n' % (each_cog_id, each_cog_id_depth, cog_id_to_description_dict[each_cog_id]))
        pwd_cog_stats_depth_handle.close()

    #################### export func_stats_copy ####################

    pwd_func_stats_copy_handle = open(pwd_func_stats_copy, 'w')
    pwd_func_stats_copy_handle.write('Category\tcopy\tDescription\n')
    for each_cog_cate in cog_category_list:
        each_cog_cate_copy = 0
        if each_cog_cate in cog_cate_num_dict:
            each_cog_cate_copy = cog_cate_num_dict[each_cog_cate]
        pwd_func_stats_copy_handle.write('%s\t%s\t%s\n' % (each_cog_cate, each_cog_cate_copy, cog_category_to_description_dict.get(each_cog_cate, 'na')))
    pwd_func_stats_copy_handle.close()

    #################### export func_stats_depth ####################

    if depth_file is not None:
        pwd_func_stats_depth_handle = open(pwd_func_stats_depth, 'w')
        pwd_func_stats_depth_handle.write('Category\tdepth\tDescription\n')
        for each_cog_cate in cog_category_list:
            each_cog_cate_depth = 0
            if each_cog_cate in cog_cate_to_gene_member_dict:
                each_cog_cate_gene_member = cog_cate_to_gene_member_dict[each_cog_cate]
                for each_gene in each_cog_cate_gene_member:
                    each_gene_depth = gene_depth_dict[each_gene]
                    each_cog_cate_depth += each_gene_depth
            each_cog_cate_depth = float("{0:.2f}".format(each_cog_cate_depth))
            pwd_func_stats_depth_handle.write('%s\t%s\t%s\n' % (each_cog_cate, each_cog_cate_depth, cog_category_to_description_dict[each_cog_cate]))
        pwd_func_stats_depth_handle.close()

    #################### get pct files ####################

    AnnotateNorm(file_in=pwd_cog_stats_copy,  skip_header=True, value_column=2, Divisor_value=len(genes_with_cog), file_out=pwd_cog_stats_copy_pct,  file_out_header='arCOG\tPercent\tDescription\n')
    AnnotateNorm(file_in=pwd_func_stats_copy, skip_header=True, value_column=2, Divisor_value=len(genes_with_cog), file_out=pwd_func_stats_copy_pct, file_out_header='Category\tPercent\tDescription\n')
    if depth_file is not None:
        AnnotateNorm(file_in=pwd_cog_stats_depth,  skip_header=True, value_column=2, Divisor_value=genes_with_cog_depth, file_out=pwd_cog_stats_depth_pct,  file_out_header='arCOG\tPercent\tDescription\n')
        AnnotateNorm(file_in=pwd_func_stats_depth, skip_header=True, value_column=2, Divisor_value=genes_with_cog_depth, file_out=pwd_func_stats_depth_pct, file_out_header='Category\tPercent\tDescription\n')
    if pct_by_all is True:
        AnnotateNorm(file_in=pwd_cog_stats_copy,  skip_header=True, value_column=2, Divisor_value=len(query_seq_list), file_out=pwd_cog_stats_copy_pct_by_all,  file_out_header='arCOG\tPercent_by_all\tDescription\n')
        AnnotateNorm(file_in=pwd_func_stats_copy, skip_header=True, value_column=2, Divisor_value=len(query_seq_list), file_out=pwd_func_stats_copy_pct_by_all, file_out_header='Category\tPercent_by_all\tDescription\n')
        if depth_file is not None:
            AnnotateNorm(file_in=pwd_cog_stats_depth,  skip_header=True, value_column=2, Divisor_value=total_depth_for_all_query_genes, file_out=pwd_cog_stats_depth_pct_by_all,  file_out_header='arCOG\tPercent_by_all\tDescription\n')
            AnnotateNorm(file_in=pwd_func_stats_depth, skip_header=True, value_column=2, Divisor_value=total_depth_for_all_query_genes, file_out=pwd_func_stats_depth_pct_by_all, file_out_header='Category\tPercent_by_all\tDescription\n')


def get_COG_annot_df(annotation_dir, stats_level, annotation_df_absolute_num, annotation_df_percentage, annotation_df_percentage_by_all, with_depth, pct_by_all):

    annotation_dir_re = '%s/*_arCOG_wd' % annotation_dir
    annotation_folder_list = [os.path.basename(file_name) for file_name in glob.glob(annotation_dir_re)]

    cog_num_dict = {}
    cog_num_pct_dict = {}
    cog_num_pct_by_all_dict = {}
    all_identified_cog = set()
    for annotation_folder in annotation_folder_list:

        annotation_folder_basename = annotation_folder.split('_arCOG_wd')[0]
        pwd_annotation_stats_file = ''
        pwd_annotation_stats_file_pct = ''
        if stats_level == 'cog_id':
            if with_depth is False:
                pwd_annotation_stats_file =             '%s/%s/%s_arcog_stats.txt'                  % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct =         '%s/%s/%s_arcog_stats_pct.txt'              % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct_by_all =  '%s/%s/%s_arcog_stats_pct_by_all.txt'       % (annotation_dir, annotation_folder, annotation_folder_basename)
            else:
                pwd_annotation_stats_file =             '%s/%s/%s_arcog_stats_depth.txt'            % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct =         '%s/%s/%s_arcog_stats_depth_pct.txt'        % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct_by_all =  '%s/%s/%s_arcog_stats_depth_pct_by_all.txt' % (annotation_dir, annotation_folder, annotation_folder_basename)

        if stats_level == 'cog_cate':
            if with_depth is False:
                pwd_annotation_stats_file =             '%s/%s/%s_func_stats.txt'                   % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct =         '%s/%s/%s_func_stats_pct.txt'               % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct_by_all =  '%s/%s/%s_func_stats_pct_by_all.txt'        % (annotation_dir, annotation_folder, annotation_folder_basename)
            else:
                pwd_annotation_stats_file =             '%s/%s/%s_func_stats_depth.txt'             % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct =         '%s/%s/%s_func_stats_depth_pct.txt'         % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct_by_all =  '%s/%s/%s_func_stats_depth_pct_by_all.txt'  % (annotation_dir, annotation_folder, annotation_folder_basename)

        current_cog_to_num_dict = {}
        for cog in open(pwd_annotation_stats_file):
            if (not cog.startswith('Category	')) and (not cog.startswith('COG	')) and (not cog.startswith('arCOG	')):
                cog_split = cog.strip().split('\t')
                if with_depth is False:
                    current_cog_to_num_dict[cog_split[0]] = int(cog_split[1])
                else:
                    current_cog_to_num_dict[cog_split[0]] = float(cog_split[1])
                all_identified_cog.add(cog_split[0])
        cog_num_dict[annotation_folder_basename] = current_cog_to_num_dict

        current_cog_to_num_pct_dict = {}
        for cog in open(pwd_annotation_stats_file_pct):
            if (not cog.startswith('Category	')) and (not cog.startswith('COG	')) and (not cog.startswith('arCOG	')):
                cog_split = cog.strip().split('\t')
                current_cog_to_num_pct_dict[cog_split[0]] = float(cog_split[1])
                all_identified_cog.add(cog_split[0])
        cog_num_pct_dict[annotation_folder_basename] = current_cog_to_num_pct_dict

        if pct_by_all is True:
            current_cog_to_num_pct_by_all_dict = {}
            for cog in open(pwd_annotation_stats_file_pct_by_all):
                if (not cog.startswith('Category	')) and (not cog.startswith('COG	')) and (not cog.startswith('arCOG	')):
                    cog_split = cog.strip().split('\t')
                    current_cog_to_num_pct_by_all_dict[cog_split[0]] = float(cog_split[1])
                    all_identified_cog.add(cog_split[0])
            cog_num_pct_by_all_dict[annotation_folder_basename] = current_cog_to_num_pct_by_all_dict

    all_identified_cog_list = sorted([i for i in all_identified_cog])

    annotation_df_absolute_num_handle = open(annotation_df_absolute_num, 'w')
    annotation_df_percentage_handle = open(annotation_df_percentage, 'w')
    annotation_df_absolute_num_handle.write('\t%s\n' % '\t'.join(all_identified_cog_list))
    annotation_df_percentage_handle.write('\t%s\n' % '\t'.join(all_identified_cog_list))
    for annotation_folder in sorted(annotation_folder_list):
        annotation_folder_basename = annotation_folder.split('_arCOG_wd')[0]
        current_cog_num_dict = cog_num_dict[annotation_folder_basename]
        current_cog_num_pct_dict = cog_num_pct_dict[annotation_folder_basename]

        current_cog_num_list = []
        current_cog_num_list_pct = []
        for identified_cog in all_identified_cog_list:

            # get num list
            identified_cog_num = 0
            identified_cog_num_pct = 0
            if identified_cog in current_cog_num_dict:
                identified_cog_num = current_cog_num_dict[identified_cog]
                identified_cog_num_pct = current_cog_num_pct_dict[identified_cog]

            current_cog_num_list.append(identified_cog_num)
            current_cog_num_list_pct.append(identified_cog_num_pct)

        # write out
        annotation_df_absolute_num_handle.write('%s\t%s\n' % (annotation_folder_basename, '\t'.join([str(i) for i in current_cog_num_list])))
        annotation_df_percentage_handle.write('%s\t%s\n' % (annotation_folder_basename, '\t'.join([str(i) for i in current_cog_num_list_pct])))
    annotation_df_absolute_num_handle.close()
    annotation_df_percentage_handle.close()

    ########## write out pct_by_all_file ##########

    if pct_by_all is True:
        annotation_df_percentage_by_all_handle = open(annotation_df_percentage_by_all, 'w')
        annotation_df_percentage_by_all_handle.write('\t%s\n' % '\t'.join(all_identified_cog_list))
        for annotation_folder in sorted(annotation_folder_list):
            annotation_folder_basename = annotation_folder.split('_arCOG_wd')[0]
            current_cog_num_pct_by_all_dict = cog_num_pct_by_all_dict[annotation_folder_basename]

            current_cog_num_list_pct_by_all = []
            for identified_cog in all_identified_cog_list:
                identified_cog_num_pct_by_all = 0
                if identified_cog in current_cog_num_pct_by_all_dict:
                    identified_cog_num_pct_by_all = current_cog_num_pct_by_all_dict[identified_cog]
                current_cog_num_list_pct_by_all.append(identified_cog_num_pct_by_all)

            # write out
            annotation_df_percentage_by_all_handle.write('%s\t%s\n' % (annotation_folder_basename, '\t'.join([str(i) for i in current_cog_num_list_pct_by_all])))
        annotation_df_percentage_by_all_handle.close()


def arCOG(args):

    file_in =           args['i']
    file_extension =    args['x']
    sequence_type =     args['m']
    depth_file =        args['d']
    pct_by_all =        args['pct_by_all']
    DB_dir =            args['db_dir']
    num_threads =       args['t']
    run_diamond =       args['diamond']
    evalue_cutoff =     args['evalue']

    ar18_fa           = '%s/ar18.fa'                 % DB_dir
    ar18_fa_diamond   = '%s/ar18.fa.dmnd'            % DB_dir
    ar18_ar14_02_csv  = '%s/ar18.ar14.02.csv'        % DB_dir
    cog_des_txt       = '%s/arCOGdef.tab'            % DB_dir
    pwd_fun_20_tab    = '%s/fun-20.tab'              % DB_dir

    ############################################ check whether db file exist ###########################################

    # check whether db file exist
    unfound_inputs = []
    for each_input in [ar18_ar14_02_csv, cog_des_txt, ar18_fa, pwd_fun_20_tab]:
        if (not os.path.isfile(each_input)) and (not os.path.isdir(each_input)):
            unfound_inputs.append(each_input)
    if len(unfound_inputs) > 0:
        for each_unfound in unfound_inputs:
            print('%s not found' % each_unfound)
        exit()

    if run_diamond is True:
        if os.path.isfile(ar18_fa_diamond) is False:
            print(datetime.now().strftime(time_format) + 'DB file for diamond not found, please refers to the help info for diamond db preparation')
            print(datetime.now().strftime(time_format) + 'Program exited!')
            exit()

    ################################################# read db into dict ################################################

    # get cog_id_to_category_dict and cog_id_to_description_dict (cognames2003-2014.tab)
    cog_category_set = set()
    cog_id_to_category_dict = dict()
    cog_id_to_description_dict = dict()
    for each_cog in open(cog_des_txt, encoding="ISO-8859-1"):
        each_cog_split = each_cog.strip().split('\t')
        cog_id = each_cog_split[0]
        cog_cate_str = each_cog_split[1]
        cog_cate_split = [i for i in cog_cate_str]
        cog_desc = each_cog_split[3]
        cog_id_to_description_dict[cog_id] = cog_desc
        cog_id_to_category_dict[cog_id] = cog_cate_split
        for each_cate in cog_cate_split:
            cog_category_set.add(each_cate)

    cog_category_list = sorted([i for i in cog_category_set])

    # get protein_to_cog_dict (ar18_ar14_02_csv)
    protein_to_cog_dict = {}
    for each_line in open(ar18_ar14_02_csv):
        each_line_split = each_line.strip().split(',')
        arcog_id = each_line_split[6]
        seq_id = each_line_split[2]
        if seq_id not in protein_to_cog_dict:
            protein_to_cog_dict[seq_id] = {arcog_id}
        else:
            protein_to_cog_dict[seq_id].add(arcog_id)

    # get cog_category_to_description_dict (fun-20.tab)
    cog_category_to_description_dict = {}
    for cog_category in open(pwd_fun_20_tab):
        cog_category_split = cog_category.strip().split('\t')
        cog_category_to_description_dict[cog_category_split[0]] = cog_category_split[2]

    ################################################## if input is file ################################################

    # if input is file
    if os.path.isfile(file_in) is True:

        # check whether depth file exist
        if depth_file is not None:
            if os.path.isfile(depth_file) is False:
                print(datetime.now().strftime(time_format) + 'specified depth file not found, program exited!')
                exit()

        print(datetime.now().strftime(time_format) + 'Running COG annotation for 1 file with %s cores' % (num_threads))

        file_in_path, file_in_basename, file_in_ext = sep_path_basename_ext(file_in)
        arCOG_worker([file_in, ar18_fa, protein_to_cog_dict, cog_id_to_category_dict, cog_id_to_description_dict,
                      cog_category_list, cog_category_to_description_dict, sequence_type, file_in_path, num_threads,
                      run_diamond, evalue_cutoff, depth_file, pct_by_all])

    ################################################ if input is folder ################################################

    # if input is folder
    else:
        if os.path.isdir(file_in) is False:
            print(datetime.now().strftime(time_format) + 'input folder not found, program exited')
            exit()
        else:
            input_file_re = '%s/*.%s' % (file_in, file_extension)
            input_file_name_list = [os.path.basename(file_name) for file_name in glob.glob(input_file_re)]

            if len(input_file_name_list) == 0:
                print(datetime.now().strftime(time_format) + 'input file not found, program exited')
                exit()

            # check whether depth file exist
            if depth_file is not None:

                if os.path.isfile(depth_file) is True:
                    print(datetime.now().strftime(time_format) + 'please provide the folder containing individual depth files (with extension .depth) for each of your input sequence file.')
                    print(datetime.now().strftime(time_format) + 'a single file (not folder) detected, program exited!')
                    exit()

                if os.path.isdir(depth_file) is False:
                    print(datetime.now().strftime(time_format) + 'provided depth folder not found, program exited!')
                    exit()

                if os.path.isdir(depth_file) is True:

                    undetected_depth_file = []
                    for input_seq_file in input_file_name_list:
                        input_seq_file_basename = '.'.join(input_seq_file.split('.')[:-1])
                        input_seq_file_depth = '%s/%s.depth' % (depth_file, input_seq_file_basename)
                        if os.path.isfile(input_seq_file_depth) is False:
                            undetected_depth_file.append(input_seq_file_depth)

                    if len(undetected_depth_file) > 0:
                        print(datetime.now().strftime(time_format) + 'the following depth files not found, program exited!')
                        print(','.join(undetected_depth_file))
                        exit()

            ################################################### define file name ###################################################

            file_in_folder_name = file_in
            if '/' in file_in:
                file_in_folder_name = file_in.split('/')[-1]

            # create output folder
            output_folder = '%s_arCOG_wd' % file_in_folder_name
            force_create_folder(output_folder)

            ######################################################### main #########################################################

            print(datetime.now().strftime(time_format) + 'Running COG annotation for %s files with %s cores' % (len(input_file_name_list), num_threads))

            list_for_multiple_arguments_COG = []
            for input_file in input_file_name_list:
                input_file_basename = '.'.join(input_file.split('.')[:-1])
                pwd_input_file = '%s/%s' % (file_in, input_file)

                # get path to current depth file
                if depth_file is None:
                    input_file_depth = None
                else:
                    input_file_depth = '%s/%s.depth' % (depth_file, input_file_basename)

                list_for_multiple_arguments_COG.append([pwd_input_file, ar18_fa, protein_to_cog_dict,
                                                        cog_id_to_category_dict, cog_id_to_description_dict,
                                                        cog_category_list, cog_category_to_description_dict,
                                                        sequence_type, output_folder, 1, run_diamond, evalue_cutoff,
                                                        input_file_depth, pct_by_all])

            # run COG annotaion files with multiprocessing
            pool = mp.Pool(processes=num_threads)
            pool.map(arCOG_worker, list_for_multiple_arguments_COG)
            pool.close()
            pool.join()

            ######################################################### get dataframe #########################################################

            annotation_df_cog_cate_copy =             '%s/%s_arCOG_cate.txt'                    % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_copy_pct =         '%s/%s_arCOG_cate_pct.txt'                % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_copy_pct_by_all =  '%s/%s_arCOG_cate_pct_by_all.txt'         % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_depth =            '%s/%s_arCOG_cate_depth.txt'              % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_depth_pct =        '%s/%s_arCOG_cate_depth_pct.txt'          % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_depth_pct_by_all = '%s/%s_arCOG_cate_depth_pct_by_all.txt'   % (output_folder, file_in_folder_name)
            annotation_df_cog_id_copy =               '%s/%s_arCOG_id.txt'                      % (output_folder, file_in_folder_name)
            annotation_df_cog_id_copy_pct =           '%s/%s_arCOG_id_pct.txt'                  % (output_folder, file_in_folder_name)
            annotation_df_cog_id_copy_pct_by_all =    '%s/%s_arCOG_id_pct_by_all.txt'           % (output_folder, file_in_folder_name)
            annotation_df_cog_id_depth =              '%s/%s_arCOG_id_depth.txt'                % (output_folder, file_in_folder_name)
            annotation_df_cog_id_depth_pct =          '%s/%s_arCOG_id_depth_pct.txt'            % (output_folder, file_in_folder_name)
            annotation_df_cog_id_depth_pct_by_all =   '%s/%s_arCOG_id_depth_pct_by_all.txt'     % (output_folder, file_in_folder_name)

            print(datetime.now().strftime(time_format) + 'Data matrix exported to:')

            # get df
            get_COG_annot_df(output_folder, 'cog_cate', annotation_df_cog_cate_copy, annotation_df_cog_cate_copy_pct, annotation_df_cog_cate_copy_pct_by_all, with_depth=False, pct_by_all=False)
            get_COG_annot_df(output_folder, 'cog_id', annotation_df_cog_id_copy, annotation_df_cog_id_copy_pct, annotation_df_cog_id_copy_pct_by_all, with_depth=False, pct_by_all=False)
            if pct_by_all is True:
                get_COG_annot_df(output_folder, 'cog_cate', annotation_df_cog_cate_copy, annotation_df_cog_cate_copy_pct, annotation_df_cog_cate_copy_pct_by_all, with_depth=False, pct_by_all=True)
                get_COG_annot_df(output_folder, 'cog_id', annotation_df_cog_id_copy, annotation_df_cog_id_copy_pct, annotation_df_cog_id_copy_pct_by_all, with_depth=False, pct_by_all=True)

            # report
            if pct_by_all is False:
                print(datetime.now().strftime(time_format) + '%s and %s'        % (annotation_df_cog_id_copy.split('/')[-1], annotation_df_cog_id_copy_pct.split('/')[-1]))
                print(datetime.now().strftime(time_format) + '%s and %s'        % (annotation_df_cog_cate_copy.split('/')[-1], annotation_df_cog_cate_copy_pct.split('/')[-1]))
            else:
                print(datetime.now().strftime(time_format) + '%s, %s and %s'    % (annotation_df_cog_id_copy.split('/')[-1], annotation_df_cog_id_copy_pct.split('/')[-1], annotation_df_cog_id_copy_pct_by_all.split('/')[-1]))
                print(datetime.now().strftime(time_format) + '%s, %s and %s'    % (annotation_df_cog_cate_copy.split('/')[-1], annotation_df_cog_cate_copy_pct.split('/')[-1], annotation_df_cog_cate_copy_pct_by_all.split('/')[-1]))

            if depth_file is not None:
                get_COG_annot_df(output_folder, 'cog_cate', annotation_df_cog_cate_depth, annotation_df_cog_cate_depth_pct, annotation_df_cog_cate_depth_pct_by_all, with_depth=True, pct_by_all=False)
                get_COG_annot_df(output_folder, 'cog_id', annotation_df_cog_id_depth, annotation_df_cog_id_depth_pct, annotation_df_cog_id_depth_pct_by_all, with_depth=True, pct_by_all=False)
                if pct_by_all is True:
                    get_COG_annot_df(output_folder, 'cog_cate', annotation_df_cog_cate_depth, annotation_df_cog_cate_depth_pct, annotation_df_cog_cate_depth_pct_by_all, with_depth=True, pct_by_all=True)
                    get_COG_annot_df(output_folder, 'cog_id', annotation_df_cog_id_depth, annotation_df_cog_id_depth_pct, annotation_df_cog_id_depth_pct_by_all, with_depth=True, pct_by_all=True)

                # report
                if pct_by_all is False:
                    print(datetime.now().strftime(time_format) + '%s and %s' % (annotation_df_cog_id_depth.split('/')[-1], annotation_df_cog_id_depth_pct.split('/')[-1]))
                    print(datetime.now().strftime(time_format) + '%s and %s' % (annotation_df_cog_cate_depth.split('/')[-1], annotation_df_cog_cate_depth_pct.split('/')[-1]))
                else:
                    print(datetime.now().strftime(time_format) + '%s, %s and %s' % (annotation_df_cog_id_depth.split('/')[-1], annotation_df_cog_id_depth_pct.split('/')[-1], annotation_df_cog_id_depth_pct_by_all.split('/')[-1]))
                    print(datetime.now().strftime(time_format) + '%s, %s and %s' % (annotation_df_cog_cate_depth.split('/')[-1], annotation_df_cog_cate_depth_pct.split('/')[-1], annotation_df_cog_cate_depth_pct_by_all.split('/')[-1]))

    ################################################## Final report ####################################################

    print(datetime.now().strftime(time_format) + 'Done!')


if __name__ == '__main__':

    COG_parser = argparse.ArgumentParser(usage=arCOG_parser_usage)
    COG_parser.add_argument('-i',               required=True,                              help='path to input sequences (in multi-fasta format)')
    COG_parser.add_argument('-x',               required=False,                             help='file extension')
    COG_parser.add_argument('-m',               required=True,                              help='sequence type, "N/n" for "nucleotide", "P/p" for "protein"')
    COG_parser.add_argument('-d',               required=False, default=None,               help='gene depth file/folder')
    COG_parser.add_argument('-pct_by_all',      required=False, action='store_true',        help='normalize by all query genes, including those without annotation')
    COG_parser.add_argument('-db_dir',          required=True,                              help='COG_db_dir')
    COG_parser.add_argument('-diamond',         required=False, action='store_true',        help='run diamond (for big dataset), default: blastp from NCBI')
    COG_parser.add_argument('-t',               required=False, type=int, default=1,        help='number of threads')
    COG_parser.add_argument('-evalue',          required=False, default=0.0001, type=float,  help='evalue cutoff, default: 0.0001')
    args = vars(COG_parser.parse_args())
    arCOG(args)
